# Copyright 2011-2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from nephoria.baseops.botobaseops import BotoBaseOps
import boto
from boto.ec2.regioninfo import RegionInfo
from boto.sts import STSConnection


class STSops(BotoBaseOps):
    SERVICE_PREFIX = 'ec2'
    EUCARC_URL_NAME = 'sts_url'
    CONNECTION_CLASS = STSConnection

    def setup_resource_trackers(self):
        pass

    def get_session_token( self, duration=None ):
        """
        Get a possibly cached session token, if getting a new token request the given duration
        Options:
            duration - The desired duration for the token in seconds (if issued, None for default duration)
        """
        return self.connection.get_session_token( duration )

    def issue_session_token( self, duration=None ):
        """
        Get a newly issued session token with the given (or default) duration
        Options:
            duration - The desired duration for the token in seconds (None for default duration)
        """
        return self.connection.get_session_token( duration, force_new=True )






#!/usr/bin/python

from logging import DEBUG, NOTSET
from boto.regioninfo import RegionInfo
from boto import set_stream_logger
from boto import __version__ as boto_version
from nephoria.baseops import BaseOps, AWSRegionData
from cloud_utils.log_utils.eulogger import Eulogger
import re

class BotoBaseOps(BaseOps):
    EUCARC_URL_NAME = None
    SERVICE_PREFIX = None
    CONNECTION_CLASS = None

    def create_connection_kwargs(self, **kwargs):
        """
        Create connection kwargs for Boto type connections. See Baseops.__init__(**kwargs)
        for kwargs available
        """
        region = kwargs.get('region')
        service_url = kwargs.get('service_url')
        validate_certs = kwargs.get('validate_certs', None)
        api_version = kwargs.get('api_version', None)
        is_secure = kwargs.get('is_secure', True)
        connection_debug = kwargs.get('connection_debug')
        region = self._get_region_info(host=self.service_host,
                                       endpoint=self.service_host,
                                       region_name=region)

        # This needs to be re-visited due to changes in Eucalyptus and Boto regarding certs...
        if validate_certs is None:
            validate_certs = True
            if re.search('2.6', boto_version):
                validate_certs = False

        self._connection_kwargs = {'service_name': self.SERVICE_PREFIX,
                                   'aws_access_key_id': self.eucarc.aws_access_key,
                                   'aws_secret_access_key': self.eucarc.aws_secret_key,
                                   'is_secure': is_secure,
                                   'use_ssl': is_secure,
                                   'port': self.service_port,
                                   'host': self.service_host,
                                   'debug': connection_debug,
                                   'region': region,
                                   'region_name': self.service_region,
                                   'verify': validate_certs,
                                   'validate_certs': validate_certs,
                                   'endpoint_url': self.service_url,
                                   'api_version': api_version,
                                   'path': self.service_path}
        return self._connection_kwargs

    def enable_connection_debug(self, level=DEBUG, format_string=None):
        try:
            self.connection.debug = 2
            level = Eulogger.format_log_level(level, 'DEBUG')
            set_stream_logger('boto', level=level, format_string=None)
        except:
            self.log.error('Could not enable debug for: "{0}"'.format(self))
            raise

    def disable_connection_debug(self, level=NOTSET):
        try:
            self.connection.debug = 0
            level = Eulogger.format_log_level(level, 'NOTSET')
            set_stream_logger('boto', level=level, format_string=None)
        except:
            self.log.error('Could not disable debug for: "{0}"'.format(self))
            raise

    def _get_region_info(self, host=None, endpoint=None, region_name=None):
        self.log.debug("get region info params: host:{0}, endpoint:{1}, "
                       "region_name:{2}".format(host, endpoint, region_name))
        if (host or endpoint or region_name):
            region = RegionInfo()
            if region_name:
                region.name = region_name
                self.log.debug("Check region: " + str(region))
                try:
                    if not endpoint:
                        endpoint_url = AWSRegionData[region_name]
                        if self.SERVICE_PREFIX and \
                                not(endpoint_url.startswith(self.SERVICE_PREFIX)):
                            endpoint_url = "{0}.{1}".format(self.SERVICE_PREFIX,
                                                            endpoint_url)
                        region.endpoint = endpoint_url
                    else:
                        region.endpoint = endpoint
                except KeyError:
                    raise Exception('Unknown region: %s' % region)
            else:
                region.name = host or endpoint
                if endpoint:
                    region.endpoint = endpoint
                elif host:
                    region.endpoint = host
            return region
        return None



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

from logging import INFO, DEBUG
from boto3.session import Session
from cloud_utils.log_utils.eulogger import Eulogger
from cloud_utils.log_utils import get_traceback, red
from cloud_admin.access.autocreds import AutoCreds
from nephoria.aws.iam.iamops import IAMops
from nephoria.aws.s3.s3ops import S3ops
from nephoria.aws.ec2.ec2ops import EC2ops
from nephoria.aws.ec2.b3_ec2ops import B3_EC2ops
from nephoria.aws.elb.elbops import ELBops
from nephoria.aws.sts.stsops import STSops
from nephoria.aws.sqs.sqsops import SQSops
from nephoria.aws.cloudformation.cfnops import CFNops
from nephoria.aws.cloudwatch.cwops import CWops
from nephoria.aws.autoscaling.asops import ASops
from nephoria import __DEFAULT_API_VERSION__

class UserContext(AutoCreds):

    # This map is used for ops connection class lookups...
    CLASS_MAP = {IAMops.__name__: 'iam',
                 S3ops.__name__: 's3',
                 EC2ops.__name__: 'ec2',
                 ELBops.__name__: 'elb',
                 STSops.__name__: 'sts',
                 SQSops.__name__: 'sqs',
                 CWops.__name__: 'cloudwatch',
                 CFNops.__name__: 'cloudformation',
                 ASops.__name__: 'autoscaling',
                 B3_EC2ops.__name__: 'b3_ec2ops'}

    def __init__(self,  aws_access_key=None, aws_secret_key=None, aws_account_name=None,
                 aws_user_name=None, port=8773, credpath=None, string=None, region=None,
                 machine=None, keysdir=None, logger=None, service_connection=None,
                 eucarc=None, existing_certs=False, boto_debug=0, https=True, api_version=None,
                 log_level=None):
        if log_level is None:
            if service_connection:
                log_level = service_connection.log.stdout_level
            else:
                log_level = DEBUG
        super(UserContext, self).__init__(aws_access_key=aws_access_key,
                                          aws_secret_key=aws_secret_key,
                                          aws_account_name=aws_account_name,
                                          aws_user_name=aws_user_name,
                                          service_port=port, region_domain=region,
                                          credpath=credpath, string=string,
                                          machine=machine, keysdir=keysdir,
                                          logger=logger, log_level=log_level,
                                          existing_certs=existing_certs,
                                          service_connection=service_connection,
                                          https=https, auto_create=False)
        self._user_info = {}
        self._session = None
        self._connections = {}
        self.region = region
        self.api_version = api_version or __DEFAULT_API_VERSION__

        # Logging setup
        if not logger:
            logger = Eulogger(str(self), stdout_level=log_level)
        self.log = logger
        self.log.debug = self.log.debug
        self.critical = self.log.critical
        self.info = self.log.info
        if eucarc:
            for key, value in eucarc.__dict__.iteritems():
                setattr(self, key, value)
        elif not (aws_access_key and aws_secret_key and self.serviceconnection):
            try:
                self.auto_find_credentials(assume_admin=False)
            except ValueError as VE:
                self.log.error('Failed to auto create user credentials and service paths:"{0}"'
                               .format(VE))
        if service_connection:
            self.update_attrs_from_cloud_services()
        self._test_resources = {}
        self._connection_kwargs = {'eucarc': self, 
                                   'connection_debug': boto_debug,
                                   'user_context': self,
                                   'region': region,
                                   'api_version': self.api_version,
                                   'log_level': log_level}
        self.log.identifier = str(self)
        self.log.debug('Successfully created User Context')

    ##########################################################################################
    #   User/Account Properties, Attributes, Methods, etc..
    ##########################################################################################

    def __repr__(self):
        account_name = ""
        user_name = ""
        account_id = ""
        try:
            account_name = self.account_name or ""
            user_name = self.user_name or ""
            account_id = self.account_id or ""
            if not (account_name or user_name or account_id) and self.access_key:
                account_id = "KEYID:{0}".format(self.access_key)
            if account_name:
                account_name = ":{0}".format(account_name)
            if user_name:
                user_name = ":{0}".format(user_name)

        except:
            pass
        return "{0}:{1}:{2}:{3}".format(self.__class__.__name__, account_id,
                                      account_name, user_name)

    @property
    def test_resources(self):
        resource_dict = {}
        for key, value in self._connections.iteritems():
            resource_dict[key] = getattr(value, 'test_resources', {})
        return resource_dict

    @property
    def user_info(self):
        if not self._user_info:
            if self.iam:
                if self.account_name == 'eucalyptus' and self.user_name == 'admin':
                    delegate_account = self.account_id
                else:
                    delegate_account = None
                self._user_info = self.iam.get_user_info(delegate_account=delegate_account)
        return self._user_info

    @property
    def user_name(self):
        if not self._user_name:
            self._user_name = self.user_info.get('user_name', None)
        return self._user_name

    @property
    def user_id(self):
        return self.user_info.get('user_id', None)

    @property
    def account_name(self):
        if not self._account_name:
            if self.iam:
                account = self.iam.get_account(account_name=self._account_name)
                self._account_name = account.get('account_name', None)
        return self._account_name

    @property
    def account_id(self):
        if not self._account_id:
            if self.iam:
                account = self.iam.get_account(account_name=self._account_name)
                self._account_id = account.get('account_id', None)
        return self._account_id

    ##########################################################################################
    #   BASE CONNECTION INFO
    ##########################################################################################

    @property
    def session(self):
        if not self._session:
            self._session = Session(aws_access_key_id=self.aws_access_key,
                                    aws_secret_access_key=self.aws_secret_key,
                                    region_name=self.region)
        return self._session

    @session.setter
    def session(self, newsession):
        if newsession is None:
            self._session = newsession
            return
        if isinstance(newsession, Session):
            self._session = newsession
            return
        raise TypeError('Unsupported type for Session, got: "{0}/{1}"'
                        .format(newsession, type(newsession)))




    ##########################################################################################
    #   CLOUD SERVICE CONNECTIONS
    ##########################################################################################

    @property
    def b3_ec2(self):
        ops_class = B3_EC2ops
        name = self.CLASS_MAP[ops_class.__name__]
        if not self._connections.get(name, None):
            self._connections[name] = ops_class(**self._connection_kwargs)
        return self._connections.get(name, None)

    @property
    def iam(self):
        ops_class = IAMops
        name = self.CLASS_MAP[ops_class.__name__]
        if not self._connections.get(name, None):
            if getattr(self, ops_class.EUCARC_URL_NAME, None):
                try:
                    self._connections[name] = ops_class(**self._connection_kwargs)
                except Exception as CE:
                    self.log.error(red('{0}\nFailed to created "{1}" interface.\n'
                                   'Connection kwargs:\n{2}\nError:{3}'
                                       .format(get_traceback(),
                                               ops_class.__name__,
                                               self._connection_kwargs,
                                               CE)))
        return self._connections.get(name, None)

    @property
    def s3(self):
        ops_class = S3ops
        name = self.CLASS_MAP[ops_class.__name__]
        if not self._connections.get(name, None):
            if getattr(self, ops_class.EUCARC_URL_NAME, None):
                try:
                    self._connections[name] = ops_class(**self._connection_kwargs)
                except Exception as CE:
                    self.log.error(red('{0}\nFailed to created "{1}" interface.\n'
                                   'Connection kwargs:\n{2}\nError:{3}'
                                       .format(get_traceback(),
                                               ops_class.__name__,
                                               self._connection_kwargs,
                                               CE)))
        return self._connections.get(name, None)

    @property
    def ec2(self):
        ops_class = EC2ops
        name = self.CLASS_MAP[ops_class.__name__]
        if not self._connections.get(name, None):
            if getattr(self, ops_class.EUCARC_URL_NAME, None):
                try:
                    self._connections[name] = ops_class(**self._connection_kwargs)
                except Exception as CE:
                    self.log.error(red('{0}\nFailed to created "{1}" interface.\n'
                                   'Connection kwargs:\n{2}\nError:{3}'
                                       .format(get_traceback(),
                                               ops_class.__name__,
                                               self._connection_kwargs,
                                               CE)))
        return self._connections.get(name, None)

    @property
    def elb(self):
        ops_class = ELBops
        name = self.CLASS_MAP[ops_class.__name__]
        if not self._connections.get(name, None):
            if getattr(self, ops_class.EUCARC_URL_NAME, None):
                try:
                    self._connections[name] = ops_class(**self._connection_kwargs)
                except Exception as CE:
                    self.log.error(red('{0}\nFailed to created "{1}" interface.\n'
                                   'Connection kwargs:\n{2}\nError:{3}'
                                       .format(get_traceback(),
                                               ops_class.__name__,
                                               self._connection_kwargs,
                                               CE)))
        return self._connections.get(name, None)

    @property
    def sts(self):
        ops_class = STSops
        name = self.CLASS_MAP[ops_class.__name__]
        if not self._connections.get(name, None):
            if getattr(self, ops_class.EUCARC_URL_NAME, None):
                try:
                    self._connections[name] = ops_class(**self._connection_kwargs)
                except Exception as CE:
                    self.log.error(red('{0}\nFailed to created "{1}" interface.\n'
                                   'Connection kwargs:\n{2}\nError:{3}'
                                       .format(get_traceback(),
                                               ops_class.__name__,
                                               self._connection_kwargs,
                                               CE)))
        return self._connections.get(name, None)

    @property
    def sqs(self):
        ops_class = SQSops
        name = self.CLASS_MAP[ops_class.__name__]
        if not self._connections.get(name, None):
            if getattr(self, ops_class.EUCARC_URL_NAME, None):
                try:
                    self._connections[name] = ops_class(**self._connection_kwargs)
                except Exception as CE:
                    self.log.error(red('{0}\nFailed to created "{1}" interface.\n'
                                       'Connection kwargs:\n{2}\nError:{3}'
                                       .format(get_traceback(),
                                               ops_class.__name__,
                                               self._connection_kwargs,
                                               CE)))
        return self._connections.get(name, None)

    @property
    def autoscaling(self):
        ops_class = ASops
        name = self.CLASS_MAP[ops_class.__name__]
        if not self._connections.get(name, None):
            if getattr(self, ops_class.EUCARC_URL_NAME, None):
                try:
                    self._connections[name] = ops_class(**self._connection_kwargs)
                except Exception as CE:
                    self.log.error(red('{0}\nFailed to created "{1}" interface.\n'
                                   'Connection kwargs:\n{2}\nError:{3}'
                                       .format(get_traceback(),
                                               ops_class.__name__,
                                               self._connection_kwargs,
                                               CE)))
        return self._connections.get(name, None)

    @property
    def cloudwatch(self):
        ops_class = CWops
        name = self.CLASS_MAP[ops_class.__name__]
        if not self._connections.get(name, None):
            if getattr(self, ops_class.EUCARC_URL_NAME, None):
                try:
                    self._connections[name] = ops_class(**self._connection_kwargs)
                except Exception as CE:
                    self.log.error(red('{0}\nFailed to created "{1}" interface.\n'
                                   'Connection kwargs:\n{2}\nError:{3}'
                                       .format(get_traceback(),
                                               ops_class.__name__,
                                               self._connection_kwargs,
                                               CE)))
        return self._connections.get(name, None)

    @property
    def cloudformation(self):
        ops_class = CFNops
        name = self.CLASS_MAP[ops_class.__name__]
        if not self._connections.get(name, None):
            if getattr(self, ops_class.EUCARC_URL_NAME, None):
                try:
                    self._connections[name] = ops_class(**self._connection_kwargs)
                except Exception as CE:
                    self.log.error(red('{0}\nFailed to created "{1}" interface.\n'
                                   'Connection kwargs:\n{2}\nError:{3}'
                                       .format(get_traceback(),
                                               ops_class.__name__,
                                               self._connection_kwargs,
                                               CE)))
        return self._connections.get(name, None)



#!/usr/bin/env python
import re
from boto.exception import BotoServerError

from nephoria.testcase_utils.cli_test_runner import CliTestRunner
from nephoria.testcase_utils.cli_test_runner import SkipTestException
from nephoria.testcontroller import TestController
import copy
import time
import random
import os.path


class GenericTemplateRun(CliTestRunner):

    _CLI_DESCRIPTION = ("Test given Cloudformation template"
                        " against Eucalyptus Cloudformation. "
                        "This script utilizes the following "
                        "Cloudformation API calls: "
                        "CreateStack, DeleteStack, DescribeStacks, "
                        "and ValidateTemplate."
                        )
    _DEFAULT_CLI_ARGS = copy.copy(CliTestRunner._DEFAULT_CLI_ARGS)

    _DEFAULT_CLI_ARGS['template_file'] = {
        'args': ['--template-file'],
        'kwargs': {'dest': 'template_file',
                   'help': 'file location containing JSON template',
                   'default': None}}

    _DEFAULT_CLI_ARGS['template_params'] = {
        'args': ['--template-parameters'],
        'kwargs': {
                   'dest': 'template_params',
                   'help': ("key=value parameters to use with "
                            "stack's template. Multiple parameters "
                            "should be space-delimited."),
                   'nargs': '?',
                   'default': None}}

    _DEFAULT_CLI_ARGS['stack_name'] = {
        'args': ['--stack-name'],
        'kwargs': {'dest': 'stack_name',
                   'help': 'Name of Cloudformation Stack Deployment',
                   'default': None}}

    _DEFAULT_CLI_ARGS['disable_rollback'] = {
        'args': ['--disable-rollback'],
        'kwargs': {'dest': 'disable_rollback',
                   'help': ("Set to True to disable rollback of "
                            "the stack if stack creation failed. "
                            "Default is False."),
                   'action': 'store_true'}}

    _DEFAULT_CLI_ARGS['timeout_min'] = {
        'args': ['--timeout-in-min'],
        'kwargs': {'dest': 'timeout',
                   'help': ("The amount of time that can pass before the "
                            "stack status becomes CREATE_FAILED; if "
                            "DisableRollback is not set or is set to "
                            "False, the stack will be rolled back."),
                   'type': int,
                   'default': '1'}}

    _DEFAULT_CLI_ARGS['capabilities'] = {
        'args': ['--capabilities'],
        'kwargs': {'dest': 'capabilities',
                   'help': ("The list of capabilities that you want to allow "
                            "in the stack. If your template contains certain "
                            "resources, you must specify the CAPABILITY_IAM "
                            "value for this parameter; otherwise, this action "
                            "returns an InsufficientCapabilities error. The "
                            "following resources require you to specify the "
                            "capabilities parameter: "
                            "`AWS::CloudFormation::Stack`_,"
                            " `AWS::IAM::AccessKey`_, "
                            "`AWS::IAM::Group`_, "
                            "`AWS::IAM::InstanceProfile`_, "
                            "`AWS::IAM::Policy`_, "
                            "`AWS::IAM::Role`_, "
                            "`AWS::IAM::User`_, and "
                            "`AWS::IAM::UserToGroupAddition`_."),
                   'choices': ['CAPABILITY_IAM'],
                   'default': None}}

    _DEFAULT_CLI_ARGS['on_failure'] = {
        'args': ['--on-failure'],
        'kwargs': {'dest': 'on_failure',
                   'help': ("Determines what action will be taken if stack "
                            "creation fails. This must be one of: "
                            "DO_NOTHING, ROLLBACK, or DELETE."),
                   'choices': ['DO_NOTHING', 'ROLLBACK', 'DELETE'],
                   'default': 'ROLLBACK'}}

    @property
    def tc(self):
        tc = getattr(self, '__tc', None)
        if not tc:
            tc = TestController(self.args.clc,
                                password=self.args.password,
                                clouduser_name=self.args.test_user,
                                clouduser_account=self.args.test_account,
                                log_level=self.args.log_level)
            setattr(self, '__tc', tc)
        return tc

    @property
    def user(self):
        user = getattr(self, '__user', None)
        if not user:
            try:
                user = self.tc.get_user_by_name(
                                    aws_account_name=self.args.test_account,
                                    aws_user_name=self.args.test_user)
            except:
                user = self.tc.create_user_using_cloudadmin(
                                    aws_account_name=self.args.test_account,
                                    aws_user_name=self.args.test_user)
            setattr(self, '__user', user)
        return user

    @property
    def stack_name(self):
        """
        Make sure stack name is set.  If stack name not passed,
        generate stack name.
        """
        stack_name = getattr(self, '__stack_name', None)
        if (
               self.args.stack_name and
               not stack_name
           ):
            stack_name = self.args.stack_name
        elif not stack_name:
            stack_name = "nephoria-stack-" + str(int(time.time()))

        setattr(self, '__stack_name', stack_name)
        return stack_name

    @property
    def on_failure(self):
        """
        Check to see if --disable-rollback is set to true.
        If so, set --on-failure to None.
        """
        if self.args.disable_rollback is True:
            on_failure = None
        else:
            on_failure = self.args.on_failure
        return on_failure

    @property
    def disable_rollback(self):
        """
        Check to see if --disable-rollback is set to true.
        If so, set to value appropriately
        """
        if self.args.disable_rollback is True:
            disable_rollback = self.args.disable_rollback
        else:
            disable_rollback = None
        return disable_rollback

    def test_validate_template(self):
        """
        Test Coverage:
            - validate provided template
        """
        self.log.debug("Validating Cloudformation Template.")
        """
        Confirm the following:
            - if --template-file is used.
            - if --template-file is used, confirm file exists.
        """
        if self.args.template_file is None:
            raise ValueError("Please pass template-file.")
        elif (
               self.args.template_file and
               os.path.exists(self.args.template_file) is False
           ):
            raise ValueError("File passed for template-file does not exist.")
        else:
            temp = open(self.args.template_file)
            temp_body = temp.read()
            temp.close()
            try:
                self.tc.user.cloudformation.validate_template(
                                                   template_body=temp_body)
                self.log.debug("Template is valid.")
            except BotoServerError as e:
                self.log.error("Error validating template: " + e.error_message)
                raise e

    def test_stack_deployment(self):
        self.log.debug("Deploy Cloudformation Stack")
        """
        If template parameter(s), store them in a list
        """
        parameters = []
        if self.args.template_params:
            for param in self.args.template_params.split():
                try:
                    k, v = param.split("=")
                    parameters.append((k, v))
                except ValueError as e:
                    self.log.error("Parameter not in key=value format")
                    raise e
        else:
            self.log.debug("No parameters passed.")
        """
        If capabilities, store in a list
        Currently, only CAPABILITY_IAM should be in the list
        """
        if self.args.capabilities:
            capabilities = self.args.capabilities.split()
        else:
            capabilities = None

        """
        Make sure timeout is set to minutes.
        Timeout will be used for stack creation.
        """
        timeout = int(time.time()) + 60*int(self.args.timeout)
        """
        Create custom tags to associate test
        with resource(s)
        """
        nephoria_job_id = self.stack_name + str(time.time())
        tags = {
                "Purpose": "nephoria test resource",
                "Test Script": "generic_template_execution.py",
                "Nephoria Job ID": nephoria_job_id
               }
        """
        Create stack passing supported arguments.
        Stack information should be returned from create_stack
        """
        temp = open(self.args.template_file)
        temp_body = temp.read()
        temp.close()
        try:
            resp = self.tc.user.cloudformation.create_stack(
                               self.stack_name,
                               template_body=temp_body,
                               parameters=parameters,
                               disable_rollback=self.disable_rollback,
                               capabilities=capabilities,
                               tags=tags,
                               on_failure=self.on_failure)
        except BotoServerError as e:
            self.log.error("Failed to create stack")
            raise e

        while True:
            """
            Confirm stack completed within timeout period
            by using describe_stack leverging
            decorrelated jitter exponential backoff for each
            request. If stack failed to create, raise error
            """
            stacks = self.tc.user.cloudformation.describe_stacks(
                                   resp.stack_name)
            events_list = stacks[0].describe_events()
            events = '\n'.join(map(str, events_list[len(events_list)::-1]))
            if stacks[0].stack_status == 'CREATE_COMPLETE':
                outputs_list = stacks[0].outputs
                outputs = '\n'.join(map(str,
                                        outputs_list[len(outputs_list)::-1]))
                self.log.debug("Stack deployment complete.")
                self.log.debug("\nStack events:\n" + events)
                self.log.debug("\nStack outputs:\n" + outputs)
                break
            elif stacks[0].stack_status == 'CREATE_FAILED':
                self.log.error("Stack deployment failed: " +
                               str(stacks[0].stack_status_reason))
                self.log.error("Stack events:\n" + events)
                raise RuntimeError("Stack deployment failed: " +
                                   str(stacks[0].stack_status_reason))
            elif int(time.time()) > timeout:
                self.log.error("Stack deployment failed "
                               "to complete in provided stack "
                               "timeout: " + str(self.args.timeout) +
                               " min.")
                self.log.error("Stack events:\n" + events)
                raise RuntimeError("Stack failed to deploy "
                                   "within timeout: " +
                                   str(self.args.timeout) +
                                   " min.")
            sleep_time = min(int(timeout),
                             random.uniform(2, 2*3))
            self.log.debug("Sleep " + str(sleep_time) +
                           " seconds before next request..")
            time.sleep(sleep_time)

    def clean_method(self):
        """
        Check to see if there are any stacks.
        If so, delete the stacks
        """
        stack_id = self.stack_name
        stacks = self.tc.user.cloudformation.describe_stacks(stack_id)
        if stacks:
            for stack in stacks:
                if (
                       self.on_failure == 'DELETE' or
                       self.on_failure == 'ROLLBACK'
                   ):
                        self.log.debug("Deleting the following stack: " +
                                       str(stack.stack_name))
                        try:
                            self.tc.user.cloudformation.delete_stack(
                                                        stack.stack_name)
                        except BotoServerError as e:
                            self.log.error("Failed to delete stack")
                            raise e
                else:
                    self.log.debug("Stack and resources not deleted.")
                    pass

if __name__ == "__main__":
    test = GenericTemplateRun()
    result = test.run()
    exit(result)



#!/usr/bin/env python
from nephoria.testcase_utils.cli_test_runner import CliTestRunner, SkipTestException
from nephoria.testcontroller import TestController
import copy
import time


"""
This is intended to demonstrate some ways to write a basic test suite.

To run this test from the command line:

## First see what CLI args are provided. Note the --sample-arg added in the test vs the default
## arguments provided by the CliTestRunner Class
prompt# python run_instances.py -h

## Run the tests
prompt# python run_instances.py --clc a.b.c.d --password mypass

## Run a subset of the tests
prompt# python run_instances.py --clc a.b.c.d --test-list 'test1, test12_skip_me'


To run this test from a python shell:
prompt# ipython
In [1]: from nephoria.cloudtests.instances.run_instances import BaseEc2Template
In [2]: test = BaseEc2Template(clc='a.b.c.d', password='mypass')
In [3]: test.run()

# Or call the method directly...
In [4]: test.test1()
"""


class BaseEc2Template(CliTestRunner):

    #####################################################################################
    # Example of how to edit, add, remove the pre-baked cli arguments provided in the base
    # CliTestRunner class...
    #####################################################################################

    _DEFAULT_CLI_ARGS = copy.copy(CliTestRunner._DEFAULT_CLI_ARGS)

    _DEFAULT_CLI_ARGS['vm_count'] = {'args': ['--vm-count'],
                                     'kwargs': {'help': 'Number of VMs to run',
                                                'default': 1,
                                                'type': int}}
    _DEFAULT_CLI_ARGS['instance_timeout'] = {
        'args': ['--instance-timeout'],
        'kwargs': {'help': 'Time to wait for an instance to run',
                   'default': 300,
                   'type': int}}

    #####################################################################################
    # Populate the most commonly needed test artifacts by using dynamic properties, rather
    # than in self.__init__()...
    #####################################################################################

    # Test controller is the primary interface for system, and cloud administration.
    @property
    def tc(self):
        tc = getattr(self, '__tc', None)
        if not tc:
            tc = TestController(self.args.clc,
                                password=self.args.password,
                                clouduser_name=self.args.test_user,
                                clouduser_account=self.args.test_account,
                                log_level=self.args.log_level)
            setattr(self, '__tc', tc)
        return tc

    # Tests should be explicit about creating and managing accounts/users in the test.
    # This is for both read-ability, as well as to avoid over using 'admin' and
    # 'eucalyptus/admin' accounts.
    @property
    def user(self):
        user = getattr(self, '__user', None)
        if not user:
            try:
                user = self.tc.get_user_by_name(aws_account_name=self.args.test_account,
                                                aws_user_name=self.args.test_user)
            except:
                user = self.tc.create_user_using_cloudadmin(aws_account_name=self.args.test_account,
                                                            aws_user_name=self.args.test_user)
            setattr(self, '__user', user)
        return user

    # Most tests need some sort of emi. get_emi() applies some standard filters, but many tests
    # will want to apply more filters per customer cli arguments to fetch an EMI of proper type
    @property
    def emi(self):
        emi = getattr(self, '__emi', None)
        if not emi:
            if self.args.emi:
                emi = self.user.ec2.get_emi(emi=self.args.emi)
            else:
                try:
                    emi = self.user.ec2.get_emi(location='cirros')
                except:
                    pass
                if not emi:
                    emi = self.user.ec2.get_emi()
            setattr(self, '__emi', emi)
        return emi

    @emi.setter
    def emi(self, value):
        setattr(self, '__emi', value)

    @property
    def keypair_name(self):
        keyname = getattr(self, '__keypairname', None)
        if not keyname:
            keyname = "{0}_{1}".format(self.__class__.__name__, int(time.time()))
        return keyname

    @property
    def keypair(self):
        key = getattr(self, '__keypair', None)
        if not key:
            key = self.user.ec2.get_keypair(key_name=self.keypair_name)
            setattr(self, '__keypair', key)
        return key

    @property
    def group(self):
        group = getattr(self, '__group', None)
        if not group:
            group = self.user.ec2.add_group("{0}_group".format(self.__class__.__name__))
            self.user.ec2.authorize_group(group, port=22, protocol='tcp')
            self.user.ec2.authorize_group(group,  protocol='icmp', port=-1)
            self.user.ec2.show_security_group(group)
            setattr(self, '__group', group)
        return group

    @group.setter
    def group(self, value):
        setattr(self, '__group', value)

    #####################################################################################
    # Create the test methods...
    #####################################################################################

    """
    def test1_<put your descriptive method name here>(self):
        '''
        <Your test description, test objective and keywords needed for categorizing this test>
        '''
        <put your test here...>
    """

    def clean_method(self):
        instances = getattr(self, 'instances', [])
        keypair = getattr(self, '__keypair', None)
        self.user.ec2.terminate_instances(instances)
        if keypair:
            self.user.ec2.delete_keypair(self.keypair)



if __name__ == "__main__":

    test = BaseEc2Template()
    result = test.run()
    exit(result)




# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2014, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: vic.iglesias@eucalyptus.com

from boto.iam import IAMConnection
from boto.exception import BotoServerError
import json
import os
import re
import urllib
import time
from cloud_utils.log_utils import markup, ForegroundColor, BackGroundColor, TextStyle
from prettytable import PrettyTable
from nephoria.baseops.botobaseops import BotoBaseOps



class IAMops(BotoBaseOps):
    EUCARC_URL_NAME = 'iam_url'
    SERVICE_PREFIX = 'iam'
    CONNECTION_CLASS = IAMConnection

    def setup_resource_trackers(self):
        ## add test resource trackers and cleanup methods...
        self.test_resources["iam_accounts"] = self.test_resources.get('iam_accounts', [])
        self.test_resources_clean_methods["iam_accounts"] = None

    def create_account(self, account_name, ignore_existing=True):
        """
        Create an account with the given name

        :param account_name: str name of account to create
        """

        params = {'AccountName': account_name}
        try:
            res = self.get_response_items('CreateAccount', params, item_marker='account')
            self.log.debug("Created account: " + account_name)
        except BotoServerError as BE:
            if not (BE.status == 409 and ignore_existing):
                raise
            res = self.get_account(account_name=account_name)
            self.log.debug("create_account(). Account already exists: " + account_name)
        self.test_resources["iam_accounts"].append(account_name)
        return res
    
    def delete_account(self, account_name, recursive=False):
        """
        Delete an account with the given name

        :param account_name: str name of account to delete
        :param recursive:
        """
        self.log.debug("Deleting account: " + account_name)
        params = {
            'AccountName': account_name,
            'Recursive': recursive
        }
        self.connection.get_response('DeleteAccount', params)

    def get_all_accounts(self, account_id=None, account_name=None, search=False):
        """
        Request all accounts, return account dicts that match given criteria

        :param account_id: regex string - to use for account_name
        :param account_name: regex - to use for account ID
        :param search: boolean - specify whether to use match or search when filtering the returned list
        :return: list of account names
        """
        if search:
            re_meth = re.search
        else:
            re_meth = re.match
        if account_id and not re.match("\d{12}", account_id):
            if not account_name:
                account_name = account_id
                account_id = None
        self.log.debug('Attempting to fetch all accounts matching- account_id:' +
                          str(account_id) + ' account_name:' + str(account_name))
        response = self.get_response_items('ListAccounts', {}, item_marker='accounts',
                                            list_marker='Accounts')
        retlist = []
        for account in response:
            if account_name is not None:
                if not search:
                    account_name = "^{0}$".format(account_name.strip())
                if not re_meth(account_name, account['account_name']):
                    continue
            if account_id is not None:
                if not search:
                    account_id = "{0}".format(account_id.strip())
                if not re_meth(account['account_id'], account_id):
                    continue
            retlist.append(account)
        return retlist

    def get_account(self, account_id=None, account_name=None, search=False):
        """
        Request a specific account, returns an account dict that matches the given criteria

        :param account_id: regex string - to use for account_name
        :param account_name: regex - to use for account ID
        :param search: boolean - specify whether to use match or search when filtering the returned list
        :return: account dict
        """
        if not (account_id or account_name):
            aliases = self.get_account_aliases()
            if aliases:
                account_name = aliases[0]
            else:
                raise ValueError('get_account(). Account id, name, or alias not found')
        accounts = self.get_all_accounts(account_id=account_id, account_name=account_name,
                                         search=search)
        if accounts:
            if len(accounts) > 1:
                raise ValueError('get_account matched more than a single account with the '
                                 'provided criteria: account_id="{0}", account_name="{1}". '
                                 'Matched:{2}'
                                 .format(account_id, account_name,
                                         ", ".join(str(x) for x in accounts)))
            else:
                return accounts[0]
        return None


    def create_user(self, user_name, path="/", delegate_account=None, ignore_existing=True):
        """
        Create a user

        :param user_name: str name of user
        :param path: str user path
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an account to operate on
        """
        if not user_name:
            # Assuming this could be part of a test, allow it but warn...
            self.log.warning('create_user(). Passed unsupported user_name:"{0}"'
                                .format(user_name))
        params = {'UserName': user_name,
                  'Path': path }
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        try:
            res = self.get_response_items('CreateUser', params, item_marker='user')
            self.log.debug('Created user:"{0}"'.format(user_name))
        except BotoServerError as BE:
            if not (BE.status == 409 and ignore_existing):
                raise
            res = self.get_user(user_name=user_name, delegate_account=delegate_account)
            self.log.debug("create_user(). User already exists: " + user_name)
        return res


    def get_user(self, user_name=None, delegate_account=None):
        params = {}
        if user_name:
            params['UserName'] = user_name
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        return self.get_response_items('GetUser', params, item_marker='user')

    
    def delete_user(self, user_name, delegate_account=None):
        """
        Delete a user

        :param user_name: str name of user
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an account to operate on
        """
        self.log.debug("Deleting user " + user_name)
        params = {'UserName': user_name}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.connection.get_response('DeleteUser', params)

    def get_users_from_account(self, path=None, user_name=None, user_id=None,
                               delegate_account=None, search=False):
        """
        Returns access that match given criteria. By default will return current account.

        :param path: regex - to match for path
        :param user_name: str name of user
        :param user_id: regex - to match for user_id
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an account to operate on
        :param search: use regex search (any occurrence) rather than match (exact same strings must occur)
        :return:
        """
        self.log.debug('Attempting to fetch all users matching- user_id:' +
                          str(user_id) + ' user_name:' + str(user_name) + " acct_name:" +
                          str(delegate_account))
        retlist = []
        params = {}
        if search:
            re_meth = re.search
        else:
            re_meth = re.match
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        response = self.get_response_items('ListUsers', params, item_marker='users',
                                               list_marker='Users')
        for user in response:
            if path is not None and not re_meth(path, user['path']):
                continue
            if user_name is not None and not re_meth(user_name, user['user_name']):
                continue
            if user_id is not None and not re_meth(user_id, user['user_id']):
                continue
            retlist.append(user)
        return retlist

    def get_all_roles_from_account(self, path_prefix=None, delegate_account=None, markers=None,
                                   search_string=None):
        self.log.debug('Attempting to fetch all roles at path:"{0}" for account:"{1}"'
                       .format(path_prefix, delegate_account))
        try:
            params = {}
            if delegate_account:
                params['DelegateAccount'] = delegate_account
            if path_prefix is not None:
                params['PathPrefix'] = path_prefix
            response = self.get_response_items('ListRoles', params, item_marker='roles',
                                               list_marker='Roles') or []
            roles = []
            if search_string:
                for role in response:
                    if re.search(search_string, str(role)):
                        roles.append(role)
            else:
                roles = response
        except:
            self.log.error('Error attempt to fetch roles using delegate_account:{0}, path:{1}, ')
        return roles

    def show_all_roles_for_account(self, roles=None, path_prefix=None, delegate_account=None,
                                   search_string=None, printme=True, print_method=None):
        roles = roles or self.get_all_roles_from_account(path_prefix=path_prefix,
                                                         delegate_account=delegate_account,
                                                         search_string=search_string)
        if not roles:
            return None
        print_method = print_method or self.log.info
        account = delegate_account or ", ".join(self.get_account_aliases())
        header_len = 20
        value_len = 80
        pt = PrettyTable([markup('ROLES FOR ACCOUNT: {0}'.format(account),
                                 markups=[TextStyle.BOLD, BackGroundColor.BG_WHITE,
                                          ForegroundColor.BLUE])])
        pt.align = "l"
        pt.border = False
        pt.padding_width = 0
        name_header = 'ROLE NAME'.ljust(header_len)
        path_header = 'PATH'.ljust(header_len)
        id_header = 'ROLE ID'.ljust(header_len)
        date_header = 'CREATE DATE'.ljust(header_len)
        policy_header = 'POLICY DOC'.ljust(header_len)
        arn_header = 'ARN'
        for role in roles:
            p_header = 'param'.ljust(header_len)
            v_header = 'value'.ljust(value_len)
            long_pt = PrettyTable([p_header, v_header])
            long_pt.align = 'l'
            long_pt.max_width[p_header] = len(p_header)
            long_pt.max_width[v_header] = len(v_header)
            long_pt.header = False
            #long_pt.border = False
            long_pt.vrules = 0
            long_pt.hrules = 0
            long_pt.padding_width = 0
            long_pt.add_row([name_header, role.get('role_name', "").ljust(value_len)])
            long_pt.add_row([id_header, role.get('role_id', "")])
            long_pt.add_row([path_header, role.get('path', "")])
            long_pt.add_row([date_header, role.get('create_date', "")])
            long_pt.add_row([arn_header, role.get('arn', "")])
            long_pt.add_row([policy_header, role.get('assume_role_policy_document', "")])
            pt.add_row([str(long_pt)])
        if printme:
            print_method("\n{0}\n".format(pt))
        else:
            return pt

    def show_all_roles(self, account_search=None, role_search=None, print_method=None):
        print_method = print_method or self.log.info
        accounts = self.get_all_accounts(account_name=account_search, search=True) or []
        buf = ""
        pt = PrettyTable(["ROLES"])
        pt.header = False
        pt.border = False
        for account in accounts:
            account_name= account.get('account_name')
            account_pt = self.show_all_roles_for_account(delegate_account=account_name,
                                                         search_string=role_search,
                                                         printme=False)
            if account_pt:
                pt.add_row(["\n{0}".format(account_pt)])
        print_method("\n{0}\n".format(pt))

    def assume_role(self, arn, session_name=None, duration=3600):
        session_name = session_name or 'nephoria_' + str(int(time.time()))
        duration = duration or 0
        params = {'DurationSeconds': duration,
                  'RoleSessionName': session_name,
                  'RoleArn': arn}
        response = self.get_response_items(action='AssumeRole', params=params)



    def show_all_accounts(self, account_name=None, account_id=None, search=False,
                          print_table=True):
        """
        Debug Method to print an account list based on given filter criteria

        :param account_name: regex - to use for account_name
        :param account_id: regex - to use for account_id
        :param search: boolean - specify whether to use match or search when filtering the returned list
        """
        pt = PrettyTable(['ACCOUNT_NAME', 'ACCOUNT_ID'])
        pt.hrules = 1
        pt.align = 'l'
        list = self.get_all_accounts(account_name=account_name,
                                     account_id=account_id,
                                     search=search)
        for account in list:
            pt.add_row([account['account_name'], account['account_id']])
        if print_table:
            self.log.info("\n" + str(pt) + "\n")
        else:
            return pt


    def show_all_groups(self, account_name=None,  account_id=None,  path=None,
                        group_name=None,  group_id=None,  search=False, print_table=True):
        """
        Print all groups in an account

        :param account_name: regex - to use for account_name
        :param account_id: regex - to use for
        :param path: regex - to match for path
        :param group_name: regex - to match for user_name
        :param group_id: regex - to match for user_id
        :param search:  boolean - specify whether to use match or search when filtering the returned list
        """
        pt = PrettyTable(['ACCOUNT:', 'GROUPNAME:', 'GROUP_ID:'])
        pt.hrules = 1
        pt.align = 'l'
        list = self.get_all_groups(account_name=account_name, account_id=account_id,
                                   path=path, group_name=group_name, group_id=group_id,
                                   search=search)
        for group in list:
            pt.add_row([group['account_name'], group['group_name'], group['group_id']])
        if print_table:
            self.log.info("\n" + str(pt) + "\n")
        else:
            return pt


    def show_all_users(self, account_name=None, account_id=None,  path=None, user_name=None,
                       user_id=None, search=False, print_table=True ):
        """
        Debug Method to print a user list based on given filter criteria

        :param account_name: regex - to use for account_name
        :param account_id: regex - to use for
        :param path: regex - to match for path
        :param user_name: regex - to match for user_name
        :param user_id: regex - to match for user_id
        :param search: boolean - specify whether to use match or search when filtering the returned list
        """
        pt = PrettyTable(['ACCOUNT:', 'USERNAME:', 'USER_ID', 'ACCT_ID'])
        pt.hrules = 1
        pt.align = 'l'
        list = self.get_all_users(account_name=account_name, account_id=account_id, path=path,
                                  user_name=user_name, user_id=user_id, search=search)
        for user in list:
            pt.add_row([user['account_name'], user['user_name'],
                        user['user_id'], user['account_id']])
        if print_table:
            self.log.info("\n" + str(pt) + "\n")
        else:
            return pt

    def get_account_aliases(self, delegate_account=None):
        params = {}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        resp = self.get_response_items('ListAccountAliases', params,
                                           item_marker='account_aliases',
                                           list_marker='AccountAliases') or []
        return resp


    def get_username_for_active_connection(self):
        """
        Helper method to show the active connections username in the case that the active
        context is not this IAMops class's connection/context.
        """
        user_info = self.get_user_info()
        return  getattr(user_info, 'user_name', None)

    def get_accountname_for_active_connection(self):
        """
        Helper method to show the active connections account name/alias in the case that the active
        context is not this IAMops class's connection/context.
        """
        aliases = self.get_account_aliases()
        if aliases:
            return aliases[0]
        return None

    def get_username_eucarc(self):
        if self.eucarc:
            return self.eucarc.user_name
        return None

    def get_accountname_eucarc(self):
        if self.eucarc:
            return self.eucarc.account_name
        return None
    
    def get_connections_accountname(self):
        """
        Get account name of current user
        """
        account_info = self.get_account()
        return  getattr(account_info, 'account_name', None)

    def get_all_users(self,  account_name=None,  account_id=None,  path=None,
                      user_name=None,  user_id=None,  search=False ):
        """
        Queries all accounts matching given account criteria, returns all access found within
        these accounts which then match the given user criteria.
        Account info is added to the user dicts

        :param account_name: regex - to use for account name
        :param account_id: regex - to use for account id
        :param path: regex - to match for path
        :param user_name: regex - to match for user name
        :param user_id: regex - to match for user id
        :param search: boolean - specify whether to use match or search when filtering the
                      returned list
        :return: List of access with account name tuples
        """
        userlist=[]
        accounts = self.get_all_accounts(account_id=account_id, account_name=account_name,
                                         search=search)
        for account in accounts:
            #if account['account_id'] == self.account_id:
            #    access =self.get_users_from_account()
            #else:
            if account.get('account_id') == self.eucarc.account_id:
                delegate_account = None
            else:
                delegate_account = account['account_name']
            users = self.get_users_from_account(path=path,
                                                user_name=user_name,
                                                user_id=user_id,
                                                delegate_account=delegate_account,
                                                search=search)
            for user in users:
                user['account_name']=account['account_name']
                user['account_id']=account['account_id']
                userlist.append(user)
        return userlist

    def get_user_policy_names(self, user_name, policy_name=None, delegate_account=None,
                              search=False, ignore_admin_err=True):
        """
        Returns list of policy names associated with a given user, and match given criteria.

        :param user_name: string - user to get policies for.
        :param policy_name: regex - to match/filter returned policies
        :param delegate_account: string - used for user lookup
        :param search: specify whether to use match or search when filtering the returned list
        :return: list of policy names
        """
        retlist = []
        params = {}
        if search:
            re_meth = re.search
        else:
            re_meth = re.match
        params = {'UserName': user_name}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        try:
            response = self.connection.get_response('ListUserPolicies', params,
                                                    list_marker='PolicyNames')
            p_names = response['list_user_policies_response']['list_user_policies_result']\
                              ['policy_names']
            for name in p_names:
                if policy_name is not None and not re_meth(policy_name, name):
                    continue
                retlist.append(name)
        except BotoServerError, BE:
            err = 'Error fetching policy for params:\n{0}: '.format(params, BE)
            if BE.status == 403 and ignore_admin_err and str(user_name).strip() == 'admin':
                self.log.debug('IGNORING: '+ err)
            else:
                self.log.critical(err)
                raise
        return retlist

    def get_user_policies(self, user_name, policy_name=None, delegate_account=None, doc=None,
                          search=False, ignore_admin_err=True):
        """
        Returns list of policy dicts associated with a given user, and match given criteria.

        :param user_name: string - user to get policies for.
        :param policy_name: regex - to match/filter returned policies
        :param delegate_account: string - used for user lookup
        :param doc: policy document to use as a filter
        :param search: boolean - specify whether to use match or search when filtering the
                                 returned list
        :param ignore_admin_err: boolean- will ignore 403 responses if the user is 'admin'
        :return:
        """
        retlist = []
        params = {}
        if search:
            re_meth = re.search
        else:
            re_meth = re.match
        try:
            names = self.get_user_policy_names(user_name, policy_name=policy_name,
                                               delegate_account=delegate_account, search=search)
            for p_name in names:
                params = {'UserName': user_name,
                          'PolicyName': p_name}
                if delegate_account:
                    params['DelegateAccount'] = delegate_account
                policy = self.connection.get_response(
                            'GetUserPolicy',
                            params,
                            verb='POST')['get_user_policy_response']['get_user_policy_result']
                if doc is not None and not re_meth(doc, policy['policy_document']):
                    continue
                retlist.append(policy)
        except BotoServerError as BE:
            err_msg = 'Error fetching policy using params:\n{0}:"{1}:{2}"'\
                .format(params, BE.status, BE.message)
            if BE.status == 403 and ignore_admin_err and str(user_name).strip() == 'admin':
                self.log.debug('IGNORING:' + str(err_msg))
            else:
                self.log.critical(err_msg)
                raise
        return retlist

    def show_user_policy_summary(self,user_name,policy_name=None,delegate_account=None,
                                 doc=None, search=False, print_table=True):
        """
        Debug method to display policy summary applied to a given user

        :param user_name: string - user to get policies for.
        :param policy_name: regex - to match/filter returned policies
        :param delegate_account: string - used for user lookup
        :param doc: policy document to use as a filter
        :param search: boolean - specify whether to use match or search when filtering the returned list
        """
        title = 'POLICIES FOR USER: {0}'.format(user_name)
        main_pt = PrettyTable([title])
        main_pt.hrules = 1
        main_pt.align = 'l'
        main_pt.max_width[title] = 120
        policies = self.get_user_policies(user_name, policy_name=policy_name,
                                          delegate_account=delegate_account, doc=doc, search=search)
        if not policies:
            main_pt.add_row(['-- No Policies --'])
        else:
            for policy in policies:
                main_pt.add_row(['POLICY NAME: "{0}" :'.format(policy['policy_name'])])
                p_doc = urllib.unquote(policy['policy_document'])
                p_json = json.loads(p_doc)
                pretty_json = (json.dumps(p_json, indent=2) or "") + "\n"
                main_pt.add_row([pretty_json])
        if print_table:
            self.log.info("\n" + str(main_pt) + "\n")
        else:
            return main_pt

    def show_user_summary(self,user_name=None, delegate_account=None, account_id=None,
                          print_table=True):
        """
        Debug method for to display euare/iam info for a specific user.

        :param user_name: string - user to get policies for.
        :param delegate_account: string - used for user lookup
        :param account_id: regex - to use for account id
        """
        if not user_name and self._user_context and self._user_context.user_name:
            user_name = self._user_context.user_name
        else:
            raise ValueError('No user_name provided or found for this connection')
        if delegate_account is None:
            account_id=self.eucarc.account_id
            delegate_account= self.get_all_accounts(account_id=account_id)[0]['account_name']
        self.log.debug('Fetching user summary for: user_name:' + str(user_name) +
                   " account:" + str(delegate_account) + " account_id:" + str(account_id))
        title = 'USER SUMMARY: user:{0}, account:{1}'.format(user_name, delegate_account)
        pt = PrettyTable([title])
        pt.align ='l'
        user_table = str(self.show_all_users(account_name=delegate_account, account_id=account_id,
                                      user_name=user_name, print_table=False)) + "\n"
        pt.add_row([user_table])

        pol_pt = self.show_user_policy_summary(user_name, delegate_account=delegate_account,
                                      print_table=False)
        new_title = str(pol_pt._field_names[0]).center(len(user_table.splitlines()[0])-4)
        new_pt = PrettyTable([new_title])
        new_pt.align[new_title] = 'l'
        new_pt.hrules = 1
        new_pt._rows = pol_pt._rows
        pt.add_row([new_pt])
        if print_table:
            self.log.info("\n" + str(pt) + "\n")
        else:
            return pt

    
    def attach_policy_user(self, user_name, policy_name, policy_json, delegate_account=None):
        """
        Attach a policy string to a user

        :param user_name: string - user to apply policy to
        :param policy_name: Name to upload policy as
        :param policy_json: Policy text
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an account to operate on
        """
        self.log.debug("Attaching the following policy to " + user_name + ":" + policy_json)
        params = {'UserName': user_name,
                  'PolicyName': policy_name,
                  'PolicyDocument': policy_json}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.connection.get_response('PutUserPolicy', params, verb='POST')
    
    def detach_policy_user(self, user_name, policy_name, delegate_account=None):
        """
        Detach a policy from user

        :param user_name: string - user to apply policy to
        :param policy_name: Name to upload policy as
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an
                                 account to operate on
        """
        self.log.debug("Detaching the following policy from " + user_name + ":" + policy_name)
        params = {'UserName': user_name,
                  'PolicyName': policy_name}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.connection.get_response('DeleteUserPolicy', params, verb='POST')

    def get_all_groups(self, account_name=None, account_id=None, path=None, group_name=None,
                       group_id=None, search=False ):
        """
        Queries all accounts matching given account criteria, returns all groups found within
        these accounts which then match the given user criteria.
        Account info is added to the group dicts

        :param account_name: regex - to use for account_name
        :param account_id: regex - to use for
        :param path: regex - to match for path
        :param group_name: regex - to match for group_name
        :param group_id: regex - to match for group_id
        :param search: boolean - specify whether to use match or search when filtering the
                                 returned list
        :return:
        """
        grouplist=[]
        accounts = self.get_all_accounts(account_id=account_id, account_name=account_name,
                                         search=search)
        for account in accounts:
            groups = self.get_groups_from_account(path=path,
                                                  group_name=group_name,
                                                  group_id=group_id,
                                                  delegate_account=account['account_name'],
                                                  search=search)
            for group in groups:
                group['account_name']=account['account_name']
                group['account_id']=account['account_id']
                grouplist.append(group)
        return grouplist

    def get_groups_from_account(self, path=None, group_name=None, group_id=None,
                                delegate_account=None, search=False):
        """
        Returns groups that match given criteria. By default will return groups from
        current account.

        :param path: regex - to match for path
        :param group_name: regex - to match for group_name
        :param group_id: regex - to match for group_id
        :param delegate_account: string - to use for delegating account lookup
        :param search: specify whether to use match or search when filtering the returned list
        :return:
        """
        self.log.debug('Attempting to fetch all groups matching- group_id:' + str(group_id) +
                   ' group_name:' + str(group_name) + " acct_name:" + str(delegate_account))
        retlist = []
        params = {}
        if search:
            re_meth = re.search
        else:
            re_meth = re.match
        if delegate_account:
            params['DelegateAccount'] = delegate_account         
        response = self.connection.get_response('ListGroups', params, list_marker='Groups')
        for group in response['list_groups_response']['list_groups_result']['groups']:
            if path is not None and not re_meth(path, group['path']):
                continue
            if group_name is not None and not re_meth(group_name, group['group_name']):
                continue
            if group_id is not None and not re_meth(group_id, group['group_id']):
                continue
            retlist.append(group)
        return retlist

    def get_users_from_group(self, group_name, delegate_account=None):
        """
        :param group_name: name of the group whose access should be returned.
        :param delegate_account: specific account name when method is being called from
                                 eucalyptus admin user.
        :return: list of access of an IAM group.
        """
        ret_list = []
        params = {}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        params['GroupName'] = group_name
        response = self.connection.get_response('GetGroup', params, list_marker='Users')
        for user in response['get_group_response']['get_group_result']['access']:
            ret_list.append(user)
        return ret_list

    def get_group_policy_names(self, group_name, policy_name=None,delegate_account=None,
                               search=False):
        """
        Returns list of policy names associated with a given group, and match given criteria.

        :param group_name: string - group to get policies for.
        :param policy_name: regex - to match/filter returned policies
        :param delegate_account: string - used for group lookup
        :param search: specify whether to use match or search when filtering the returned list
        :return: list of policy names
        """
        retlist = []
        params = {}
        if search:
            re_meth = re.search
        else:
            re_meth = re.match
        params = {'GroupName': group_name}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        response = self.connection.get_response('ListGroupPolicies',
                                                params, list_marker='PolicyNames')
        for name in response['list_group_policies_response']['list_group_policies_result']\
                ['policy_names']:
            if policy_name is not None and not re_meth(policy_name, name):
                continue
            retlist.append(name)
        return retlist

    def get_group_policies(self, group_name, policy_name=None,delegate_account=None, doc=None,
                           search=False):
        """
        Returns list of policy dicts associated with a given group, and match given criteria.

        :param group_name: string - group to get policies for.
        :param policy_name: regex - to match/filter returned policies
        :param delegate_account: string - used for group lookup
        :param doc: policy document to use as a filter
        :param search: boolean - specify whether to use match or search when filtering the
                                 returned list
        :return:
        """
        retlist = []
        params = {}
        if search:
            re_meth = re.search
        else:
            re_meth = re.match
        names = self.get_group_policy_names(group_name, policy_name=policy_name,
                                            delegate_account=delegate_account, search=search)

        for p_name in names:
            params = {'GroupName': group_name,
                      'PolicyName': p_name}
            if delegate_account:
                params['DelegateAccount'] = delegate_account
            policy = self.connection.get_response('GetGroupPolicy', params, verb='POST')\
                ['get_group_policy_response']['get_group_policy_result']
            if doc is not None and not re_meth(doc, policy['policy_document']):
                continue
            retlist.append(policy)
        return retlist
    
    def create_group(self, group_name,path="/", delegate_account=None):
        """
        Create group.

        :param
        :param path: path for group
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an
                                 account to operate on
        """
        self.log.debug("Attempting to create group: " + group_name)
        params = {'GroupName': group_name,
                  'Path': path}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.connection.get_response('CreateGroup', params)
    
    def delete_group(self, group_name, delegate_account=None):
        """
        Delete group.

        :param group_name: name of group to delete
        :param delegate_account:
        """
        self.log.debug("Deleting group " + group_name)
        params = {'GroupName': group_name}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.connection.get_response('DeleteGroup', params)
    
    def add_user_to_group(self, group_name, user_name, delegate_account=None):
        """
        Add a user to a group.

        :param group_name: name of group to add user to
        :param user_name: name of user to add to group
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an
                                 account to operate on
        """
        self.log.debug("Adding user "  +  user_name + " to group " + group_name)
        params = {'GroupName': group_name,
                  'UserName': user_name}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.connection.get_response('AddUserToGroup', params)
    
    def remove_user_from_group(self, group_name, user_name, delegate_account=None):
        """
        Remove a user from a group.

        :param group_name: name of group to remove user from
        :param user_name: name of user to remove from group
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an
                                 account to operate on
        """
        self.log.debug("Removing user "  +  user_name + " to group " + group_name)
        params = {'GroupName': group_name,
                  'UserName': user_name}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.connection.get_response('RemoveUserFromGroup', params)
    
    def attach_policy_group(self, group_name, policy_name, policy_json, delegate_account=None):
        """
        Attach a policy to a group.

        :param group_name: name of group to remove user from
        :param policy_name: Name to upload policy as
        :param policy_json: Policy text
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an
                                 account to operate on
        """
        self.log.debug("Attaching the following policy to " + group_name + ":" + policy_json)
        params = {'GroupName': group_name,
                  'PolicyName': policy_name,
                  'PolicyDocument': policy_json}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.connection.get_response('PutGroupPolicy', params, verb='POST')
    
    def detach_policy_group(self, group_name, policy_name, delegate_account=None):
        """
        Remove a policy from a group.

        :param group_name: name of group to remove user from
        :param policy_name: Name to upload policy as
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an
                                 account to operate on
        """
        self.log.debug("Detaching the following policy from " + group_name + ":" + policy_name)
        params = {'GroupName': group_name,
                  'PolicyName': policy_name}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        self.connection.get_response('DeleteGroupPolicy', params, verb='POST')
    
    def create_access_key(self, user_name=None, delegate_account=None):
        """
        Create a new access key for the user.

        :param user_name: Name of user to create access key for to
        :param delegate_account: str can be used by Cloud admin in Eucalyptus to choose an
                                 account to operate on
        :return: A tuple of access key and and secret key with keys: 'access_key_id' and
                'secret_access_key'
        """
        self.log.debug("Creating access key for " + user_name )
        params = {'UserName': user_name}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        response = self.connection.get_response('CreateAccessKey', params)
        access_tuple = {}
        access_tuple['access_key_id'] = response['create_access_key_response']\
            ['create_access_key_result']['access_key']['access_key_id']
        access_tuple['secret_access_key'] = response['create_access_key_response']\
            ['create_access_key_result']['access_key']['secret_access_key']
        return access_tuple

    def get_aws_access_key(self, user_name=None, delegate_account=None):
        if not user_name and not delegate_account and self.connection.aws_access_key_id:
            aws_access_key = self.connection.aws_access_key_id or self.eucarc.aws_access_key
            if aws_access_key:
                return  aws_access_key
        params = {}
        if user_name:
            params['UserName'] = user_name
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        response = self.get_response_items('ListAccessKeys', params, item_marker='member')
        #result = response['list_access_keys_response']['list_access_keys_result']
        #return result['access_key_metadata']['member']['access_key_id']
        return response

    def create_signing_cert(self, user_name=None, delegate_account=None):
        params = {}
        if user_name:
            params['UserName'] = user_name
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        response = self.get_response_items('CreateSigningCertificate', params,
                                           item_marker='certificate')

    def delete_signing_cert(self, cert_id, user_name=None, delegate_account=None):
        params = {'CertificateId': cert_id}
        if user_name:
            params['UserName'] = user_name
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        return self.connection.get_response('DeleteSigningCertificate', params)

    def delete_all_signing_certs(self, user_name=None, delegate_account=None, verbose=False):
        for cert in self.get_all_signing_certs(user_name=user_name,
                                               delegate_account=delegate_account):
            certid = cert.get('certificate_id')
            if certid:
                if verbose:
                    self.log.debug('Deleting signing cert: "{0}"'.format(cert))
                self.delete_signing_cert(certid, user_name=user_name,
                                         delegate_account=delegate_account)
            else:
                raise ValueError('certificate_id not found for cert dict:"{0}"'.format(cert))


    def get_all_signing_certs(self, marker=None, max_items=None,
                              user_name=None, delegate_account=None):
        params = {}
        if marker:
            params['Marker'] = marker
        if max_items:
            params['MaxItems'] = max_items
        if user_name:
            params['UserName'] = user_name
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        return self.get_response_items('ListSigningCertificates',
                                       params, item_marker='certificates',
                                       list_marker='Certificates')


    def get_active_id_for_cert(self, certpath, machine):
        '''
        Attempt to get the cloud's active id for a certificate at 'certpath' on
        the 'machine' filesystem. Also see is_ec2_cert_active() for validating the current
        cert in use or the body (string buffer) of a cert.
        :param certpath: string representing the certificate path on the machines filesystem
        :param machine: Machine obj which certpath exists on
        :returns :str() certificate id (if cert is found to be active) else None
        '''
        if not certpath:
            raise ValueError('No ec2 certpath provided or set for eutester obj')
        self.log.debug('Verifying cert: "{0}"...'.format(certpath))
        body = str("\n".join(machine.sys('cat {0}'.format(certpath), verbose=False)) ).strip()
        certs = []
        if body:
            certs = self.get_all_signing_certs()
        for cert in certs:
            if str(cert.get('certificate_body')).strip() == body:
                self.log.debug('verified certificate with id "{0}" is still valid'
                           .format(cert.get('certificate_id')))
                return cert.get('certificate_id')
        self.log.debug('Cert: "{0}" is NOT active'.format(certpath or body))
        return None

    def find_active_cert_and_key_in_dir(self, machine, dir="", recursive=True):
        '''
        Attempts to find an "active" cert and the matching key files in the provided
        directory 'dir' on the provided 'machine' via ssh.
        If recursive is enabled, will attempt a recursive search from the provided directory.
        :param dir: the base dir to search in on the machine provided
        :param machine: a Machine() obj used for ssh search commands
        :param recursive: boolean, if set will attempt to search recursively from the dir provided
        :returns dict w/ values 'certpath' and 'keypath' or {} if not found.
        '''
        ret_dict = {}
        if dir and not dir.endswith("/"):
            dir += "/"
        if recursive:
            rec = "r"
        else:
            rec = ""
        certfiles = machine.sys('grep "{0}" -l{1} {2}*.pem'.format('^-*BEGIN CERTIFICATE', rec, dir))
        for f in certfiles:
            if self.get_active_id_for_cert(f, machine=machine):
                dir = os.path.dirname(f)
                keypath = self.get_key_for_cert(certpath=f, keydir=dir, machine=machine)
                if keypath:
                    self.log.debug('Found existing active cert and key on clc: {0}, {1}'
                                      .format(f, keypath))
                    return {'certpath':f, 'keypath':keypath}
        return ret_dict

    def get_key_for_cert(self, certpath, keydir, machine, recursive=True):
        '''
        Attempts to find a matching key for cert at 'certpath' in the provided directory 'dir'
        on the provided 'machine'.
        If recursive is enabled, will attempt a recursive search from the provided directory.
        :param dir: the base dir to search in on the machine provided
        :param machine: a Machine() obj used for ssh search commands
        :param recursive: boolean, if set will attempt to search recursively from the dir provided
        :returns string representing the path to the key found or None if not found.
        '''
        self.log.debug('Looking for key to go with cert...')
        if keydir and not keydir.endswith("/"):
            keydir += "/"
        if recursive:
            rec = "r"
        else:
            rec = ""
        certmodmd5 = machine.sys('openssl x509 -noout -modulus -in {0}  | md5sum'
                                 .format(certpath))
        if certmodmd5:
            certmodmd5 = str(certmodmd5[0]).strip()
        else:
            return None
        keyfiles = machine.sys('grep "{0}" -lz{1} {2}*.pem'
                               .format("^\-*BEGIN RSA PRIVATE KEY.*\n.*END RSA PRIVATE KEY\-*",
                                       rec, keydir))
        for kf in keyfiles:
            keymodmd5 = machine.sys('openssl rsa -noout -modulus -in {0} | md5sum'.format(kf))
            if keymodmd5:
                keymodmd5 = str(keymodmd5[0]).strip()
            if keymodmd5 == certmodmd5:
                self.log.debug('Found key {0} for cert {1}'.format(kf, certpath))
                return kf
        return None

    def is_ec2_cert_active(self, certbody=None):
        '''
        Attempts to verify if the current self.ec2_cert @ self.ec2_certpath is still active.
        :param certbody
        :returns the cert id if found active, otherwise returns None
        '''
        certbody = certbody or self.ec2_cert
        if not certbody:
            raise ValueError('No ec2 cert body provided or set for eutester to check for active')
        if isinstance(certbody, dict):
            checkbody = certbody.get('certificate_body')
            if not checkbody:
                raise ValueError('Invalid certbody provided, did not have "certificate body" attr')
        for cert in self.get_all_signing_certs():
            body = str(cert.get('certificate_body')).strip()
            if body and body == str(certbody).strip():
                return cert.get('certificate_id')
        return None


    def upload_server_cert(self, cert_name, cert_body, private_key):
        self.log.debug("uploading server certificate: " + cert_name)
        self.upload_server_cert(cert_name=cert_name, cert_body=cert_body,
                                           private_key=private_key)
        if cert_name not in str(self.connection.get_server_certificate(cert_name)):
            raise Exception("certificate " + cert_name + " not uploaded")

    def update_server_cert(self, cert_name, new_cert_name=None, new_path=None):
        self.log.debug("updating server certificate: " + cert_name)
        self.connection.update_server_cert(cert_name=cert_name,
                                                          new_cert_name=new_cert_name,
                                                          new_path=new_path)
        if (new_cert_name and new_path) not in \
                str(self.connection.get_server_certificate(new_cert_name)):
            raise Exception("certificate " + cert_name + " not updated.")

    def get_server_cert(self, cert_name):
        self.log.debug("getting server certificate: " + cert_name)
        cert = self.connection.get_server_certificate(cert_name=cert_name)
        self.log.debug(cert)
        return cert

    def delete_server_cert(self, cert_name):
        self.log.debug("deleting server certificate: " + cert_name)
        self.connection.delete_server_cert(cert_name)
        if (cert_name) in str(self.connection.get_all_server_certs()):
            raise Exception("certificate " + cert_name + " not deleted.")

    def list_server_certs(self, path_prefix='/', marker=None, max_items=None):
        self.log.debug("listing server certificates")
        certs = self.connection.list_server_certs(path_prefix=path_prefix,
                                                      marker=marker, max_items=max_items)
        self.log.debug(certs)
        return certs

    def create_login_profile(self, user_name, password, delegate_account=None):
        self.log.debug("Creating login profile for: " + user_name + " with password: " + password)
        params = {'UserName': user_name,
                  'Password': password}
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        return self.connection.get_response('CreateLoginProfile', params, verb='POST')

    @staticmethod
    def _search_dict(dictionary, marker):
        if marker in dictionary.keys():
            return dictionary.get(marker)
        else:
            for value in dictionary.itervalues():
                if isinstance(value, dict):
                    res = IAMops._search_dict(value, marker)
                    if res:
                        return res
        return {}

    def get_response_items(self, action, params, item_marker, path='/', parent=None,
                               verb='POST', list_marker='Set'):
        if list_marker is None:
            list_marker = 'Set'
        resp = self.connection.get_response(action=action, params=params, path=path, parent=parent,
                                 verb=verb, list_marker=list_marker)
        return IAMops._search_dict(resp, item_marker)

    def get_user_info(self, user_name=None, delegate_account=None):
        params = {}
        if user_name:
            params['UserName'] = user_name
        if delegate_account:
            params['DelegateAccount'] = delegate_account
        return self.get_response_items(action='GetUser', params=params,
                                                  item_marker='user', list_marker='user')


class IAMResourceNotFoundException(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: matt.clark@eucalyptus.com
'''
Created on Mar 7, 2012
@author: clarkmatthew
Place holder for volume test specific convenience methods+objects to extend boto's volume class

'''
import re
from boto.ec2.volume import Volume
from boto.exception import EC2ResponseError
import time
from nephoria.euca.taggedresource import TaggedResource
from datetime import datetime, timedelta
from prettytable import PrettyTable



class EuVolume(Volume, TaggedResource):
    tag_md5_key = 'md5'
    tag_md5len_key = 'md5len'
    tag_instance_id_key = 'instance_id'
    tag_guestdev_key = 'guestdev'

    '''
    Note: Different hypervisors will honor the requested cloud dev differently, so the requested device can not 
    be relied up as the device it attached to on the guest 'guestdev'
    '''

        
    @classmethod
    def make_euvol_from_vol(cls, volume, ec2ops=None, cmdstart=None):
        newvol = EuVolume(volume.connection)
        newvol.__dict__ = volume.__dict__
        newvol.ec2ops = ec2ops
        newvol.guestdev = "" #the guest device name in use by this attached volume
        newvol.clouddev = "" #the device name given to the cloud as a request to be used.
        newvol.md5 = None
        newvol.md5len = 1024
        newvol.eutest_failmsg = None
        newvol.eutest_laststatus = newvol.status
        newvol.eutest_ageatstatus = 0 
        newvol.eutest_cmdstart = cmdstart or newvol.get_volume_time_created(volume)
        newvol.eutest_createorder = None
        newvol.eutest_cmdtime = None
        newvol.eutest_attached_instance_id = None
        if newvol.tags.has_key(newvol.tag_md5_key):
            newvol.md5 = newvol.tags[newvol.tag_md5_key]
        if newvol.tags.has_key(newvol.tag_md5len_key):
            newvol.md5len = newvol.tags[newvol.tag_md5len_key]
        newvol.set_attached_status()

        return newvol

    # imported from ec2ops
    def get_volume_time_created(self, volume):
        """
        Get the seconds elapsed since the volume was created.

        :type volume: boto volume object
        :param volume: The volume used to calculate the elapsed time since created.

        :rtype: integer
        :returns: The number of seconds elapsed since this volume was created.
        """
        last_status = volume.status
        try:
            volume.update()
        except EC2ResponseError as ER:
            if ER.status == 400 and last_status in ['deleted', 'deleting']:
                self.status = 'deleted'
                self.attach_data = None
            else:
                raise ER
        #get timestamp from attach_data
        create_time = self.get_datetime_from_resource_string(volume.create_time)
        #return the elapsed time in seconds
        return time.mktime(datetime.utcnow().utctimetuple()) - time.mktime(create_time.utctimetuple())

    # imported from ec2ops
    @staticmethod
    def get_datetime_from_resource_string(timestamp,
                                          time_format="%Y %m %d %H %M %S"):
        """
        Convert a typical resource timestamp to datetime time_struct.

        :type timestamp: string
        :param timestamp: Timestamp held within specific boto resource objects.
                          Example timestamp format: 2012-09-19T21:24:03.864Z

        :rtype: time_struct
        :returns: The time_struct representation of the timestamp provided.
        """
        t = re.findall('\w+',str(timestamp).replace('T',' '))
        #remove milliseconds from list...
        t.pop()
        #create a time_struct out of our list
        return datetime.strptime(" ".join(t), time_format)

    def update(self):
        last_status = self.status
        try:
            super(EuVolume, self).update()
            if (self.tags.has_key(self.tag_md5_key) and
                    (self.md5 != self.tags[self.tag_md5_key])) or \
                    (self.tags.has_key(self.tag_md5len_key) and
                         (self.md5len != self.tags[self.tag_md5len_key])):
                self.update_volume_attach_info_tags()
        except EC2ResponseError as ER:
            if ER.status == 400 and last_status in ['deleted', 'deleting']:
                self.status = 'deleted'
                self.attach_data =  None
            else:
                raise ER
        self.set_last_status()
    
    def set_last_status(self,status=None):
        self.eutest_laststatus = status or self.status
        self.eutest_laststatustime = time.time()
        self.set_attached_status()
        self.eutest_ageatstatus = "{0:.2f}".format(time.time() - self.eutest_cmdstart)

    def set_attached_status(self):
        if self.attach_data:
            self.eutest_attached_status = self.attach_data.status
            self.eutest_attached_instance_id = self.attach_data.instance_id
            if self.tags.has_key(self.tag_instance_id_key) and self.tags[self.tag_instance_id_key] != self.eutest_attached_instance_id:
                self.remove_tag(self.tag_instance_id_key)
                self.remove_tag(self.tag_guestdev_key)
            else:
                if not self.guestdev and self.tags.has_key(self.tag_guestdev_key):
                    self.guestdev = self.tags[self.tag_guestdev_key]
        else:
            self.eutest_attached_status = None
            self.eutest_attached_instance_id = None

    def printself(self, printmethod=None, printme=True):
        pt = PrettyTable(['VOL_ID', 'ORDER', 'TESTSTATUS', 'AGE', 'SIZE',
                          'SRC_SNAP', 'MD5/(LEN)', 'ZONE', 'INSTANCE'])
        pt.padding_width=0
        instance_id = None
        if self.attach_data:
           instance_id = self.attach_data.instance_id
        pt.add_row([self.id, self.eutest_createorder, self.eutest_laststatus or self.status,
                    self.eutest_ageatstatus, self.size, self.snapshot_id,
                    "{0}/({1})".format(self.md5, self.md5len), self.zone, instance_id])
        if printme:
            printmethod = printmethod or self.debug
            printmethod(str(pt))
        else:
            return pt

    def update_volume_attach_info_tags(self, md5=None, md5len=None, instance_id=None, guestdev=None):
        md5 = md5 or self.md5
        md5len = md5len or self.md5len
        self.add_tag(self.tag_md5_key, md5)
        self.add_tag(self.tag_md5len_key, md5len)
        if self.status == 'in-use' and hasattr(self,'attach_data') and self.attach_data:
            instance_id = instance_id or self.eutest_attached_instance_id
            guestdev = guestdev or self.guestdev
            self.add_tag(self.tag_instance_id_key, instance_id)
            self.add_tag(self.tag_guestdev_key, guestdev)
        else:
            self.set_volume_detached_tags()


    def set_volume_detached_tags(self):
        self.remove_tag(self.tag_instance_id_key)
        self.remove_tag(self.tag_guestdev_key)
    
        
        
        
        
        
        



# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: matt.clark@eucalyptus.com

import re
import time
import httplib
from xml.etree import ElementTree
import sys
from cloud_utils.net_utils.sshconnection import SshCbReturn
from cloud_utils.system_utils.machine import Machine
from cloud_utils.log_utils.eulogger import Eulogger
from cloud_utils.log_utils import TextStyle, ForegroundColor, BackGroundColor, markup, \
    get_traceback
from nephoria.aws.ec2.conversiontask import ConversionTask
from nephoria.usercontext import UserContext


class Euca2oolsImageUtils(object):
    #Define the bytes per gig
    gig = 1073741824
    mb = 1048576
    kb = 1024
    def __init__(self, access_key=None, secret_key=None, account_id=None, region_domain=None,
                 s3_url=None, ec2_url=None, bootstrap_url=None, ec2_cert_path=None,
                 worker_hostname=None, worker_keypath=None, worker_username='root',
                 worker_password=None, worker_machine=None, user_context=None, log_level='debug',
                 destpath=None, time_per_gig=300, eof=True):
        
        self.access_key = access_key
        self.secret_key = secret_key
        self.account_id = account_id
        self.region_domain = region_domain
        self.bootstrap_url = bootstrap_url
        self.s3_url = s3_url
        self.ec2_url = ec2_url
        self.ec2_cert_path = ec2_cert_path

        self.log = Eulogger('Euca2oolsImageUtils', stdout_level=log_level)
        # Setup the work machine, this is the machine which will be used for
        # performing the 'work' (download, bundle, etc)
        self.worker_hostname = worker_hostname
        self.worker_keypath = worker_keypath
        self.worker_username = worker_username
        self.worker_password = worker_password
        self._worker_machine = None
        self._user_context = user_context
        if worker_machine:
            self._worker_machine = worker_machine
        
        self.time_per_gig = time_per_gig
        self.eof = eof
        
        if destpath is not None:
            self.destpath = str(destpath)
        else:
            self.destpath = "/disk1/storage"

        self.time_per_gig = time_per_gig
        
    def status_log(self, msg):
        return self.log.info(markup(msg, 
                                    markups=[ForegroundColor.WHITE, 
                                             BackGroundColor.BG_GREEN, 
                                             TextStyle.BOLD]))
        
        
    @property
    def worker_machine(self):
        '''
        Attempts to verify the worker passed is a Machine() class else assume
        it's a host name of the machine work should be performed on and
        attempt to return a Machine() created from the hostname

        param: worker Machine() or 'hostname' to be used to perform image utils
        work on.
        returns Machine obj
        '''
        if not self._worker_machine:     
            if self.worker_hostname:
                self.log.debug('Attempting to connect to machine: "{0}" for image utility work...'
                               .format(self.worker_hostname))
                self._worker_machine = Machine(hostname=self.worker_hostname,
                                               username=self.worker_username,
                                               password=self.worker_password,
                                               keypath=self.worker_keypath)
        return self._worker_machine
    
    @worker_machine.setter
    def worker_machine(self, machine):
        if isinstance(machine, Machine):
                self._worker_machine = machine
        else:
            raise ValueError('worker_machine must be of type Machine, got:"{0}/{1}"'
                             .format(machine, type(machine)))

    def create_user_context(self, access_key, secret_key, account_id=None,
                            region_domain=None, ec2_url=None, s3_url=None, bootstrap_url=None):
        if not (region_domain or s3_url or ec2_url):
            raise ValueError('Can not derive service endpoints for user. '
                             'Must supply either region_domain:"{0}", or ec2_url:"{1}" '
                             's3_url:"{2}"'.format(region_domain, ec2_url, s3_url))
        access_key = access_key or self.access_key
        secret_key = secret_key or self.secret_key
        region_domain = region_domain or self.region_domain
        if (not access_key and secret_key and region_domain):
            raise ValueError('Must supply access_key, secret_key and region domain to '
                             'create user context')
        user = UserContext(aws_access_key=access_key, aws_secret_key=secret_key,
                           region=region_domain)
        if ec2_url:
            user.ec2_url = ec2_url
        if s3_url:
            user.s3_url = s3_url
        if bootstrap_url:
            user.bootstrap_url = bootstrap_url
        if account_id:
            user.account_id = account_id
        return user

    @property
    def user_context(self):
        if not self._user_context:
            try:
                self._user_context = self.create_user_context(access_key=self.access_key,
                                                             secret_key=self.secret_key,
                                                             region_domain=self.region_domain,
                                                             ec2_url=self.ec2_url,
                                                             s3_url=self.s3_url,
                                                             bootstrap_url=self.bootstrap_url,
                                                             account_id=self.account_id)
            except ValueError as VE:
                self.log.warning('Could not create user context, err:"{0}"'.format(VE))
        return self._user_context

    @user_context.setter
    def user_context(self, user_context):
        if user_context is None or isinstance(user_context, UserContext):
            if user_context:
                self._user_context = user_context
                self.access_key = user_context.access_key
                self.secret_key = user_context.secret_key
                self.ec2_url = user_context.ec2_url
                self.s3_url = user_context.s3_url
                self.bootstrap_url = user_context.bootstrap_url
                self.account_id = user_context.account_id
        else:
            raise ValueError('Usercontext must be of type UserContext. Got:"{0}/{1}"'
                             .format(user_context, type(user_context)))



    def getHttpRemoteImageSize(self, url, unit=None, maxredirect=5):
        return Euca2oolsImageUtils._getHttpRemoteImageSize(url, unit=unit, maxredirect=maxredirect)

    @staticmethod
    def _getHttpRemoteImageSize(url, unit=None, maxredirect=5, debug=None):
        '''
        Get the remote file size from the http header of the url given
        Returns size in GB unless unit is given.
        '''
        unit = unit or 1073741824
        if debug is None:
            def printdebug(msg):
                print msg
            debug = printdebug

        def get_location(url, depth, maxdepth):
            if depth > maxdepth:
                raise ValueError('Max redirects limit has been reached:{0}/{1}'
                                 .format(depth, maxdepth))
            conn = None
            try:
                url = url.replace('http://', '')
                host = url.split('/')[0]
                path = url.replace(host, '')
                res = None
                err = None
                retries = 5
                for retry in xrange(0, retries):
                    try:
                        debug('HTTP HEAD request for: {0}, attempt:{1}/{2}'
                              .format(url, retry, retries))
                        conn = httplib.HTTPConnection(host)
                        conn.request("HEAD", path)
                        res = conn.getresponse()
                        break
                    except Exception as HE:
                        err = '{0}\nError attempting to fetch url:{1}, attempt:{2}/{3}, ' \
                              'error:{4}'.format(get_traceback(), url, retry, retries, HE)
                        debug(err)
                        time.sleep(retry)
                if not res:
                    err = err or "Error retrieving url:{0}".format(url)
                    raise RuntimeError(err)
                location = res.getheader('location')
                if location and location != url:
                    depth += 1
                    debug('Redirecting: depth:{0}, url:{1}'.format(depth, location))
                    return get_location(location, depth=depth,maxdepth=maxdepth)
                else:
                    content_length = res.getheader('content-length')
                    if content_length is None:
                        raise ValueError('No content-length header found for url:{0}'
                                         .format(url))
                    fbytes = int(content_length)
                    return fbytes
            except Exception as HE:
                debug('Failed to fetch content-length header from url:{0}'.format(url))
                raise HE
            finally:
                if conn:
                    conn.close()
        try:
            fbytes = get_location(url, depth=0, maxdepth=maxredirect)
            debug("content-length:" + str(fbytes))
            if fbytes == 0:
                rfsize = 0
            else:
                rfsize = (((fbytes/unit) + 1) or 1)
            debug("Remote file size: " + str(rfsize) + "g")
        except Exception, e:
            debug("Failed to get remote file size...")
            raise e
        return rfsize

    def wget_image(self,
                   image_url,
                   destpath=None,
                   dest_file_name=None,
                   machine=None,
                   user=None,
                   password=None,
                   retryconn=True,
                   time_per_gig=300):
        '''
        Attempts to wget a url to a remote (worker) machine.
        :param image_url: url to wget/download
        :param destpath:path/dir to download to
        :param dest_file_name: filename to download image to
        :param machine: remote (worker) machine to wget on
        :param user: wget user name
        :param password: wget password
        :param retryconn: boolean to retry connection
        :param time_per_gig: int time to allow per gig of image wget'd
        :returns int size of image
        '''
        machine = machine or self.worker_machine
        if destpath is None and self.destpath is not None:
            destpath = self.destpath
        size = self.getHttpRemoteImageSize(image_url)
        if (size <  machine.get_available(destpath, unit=self.__class__.gig)):
            raise Exception("Not enough available space at: " +
                            str(destpath) + ", for image: " + str(image_url))
        timeout = size * time_per_gig
        self.log.debug('wget_image: ' + str(image_url) + ' to destpath' +
                       str(destpath) + ' on machine:' + str(machine.hostname))
        saved_location = machine.wget_remote_image(image_url,
                                                   path=destpath,
                                                   dest_file_name=dest_file_name,
                                                   user=user,
                                                   password=password,
                                                   retryconn=retryconn,
                                                   timeout=timeout)
        return (size, saved_location)


    def get_manifest_obj(self, path, machine=None, local=False, timeout=30):
        '''
        Read in a local or remote manifest xml file and convert to an
        xml ElementTree object.
        :param path: local or remote path to manifest file
        :param machine: the remote machine to read manifest file from
        :param local: boolean to determine if file is local
        :param timeout: timeout in second for reading in file
        :returns xml ElementTree obj
        '''
        cmd = 'cat ' + str(path)
        if not local:
            machine = machine or self.worker_machine
            out = machine.cmd(cmd, timeout=timeout, verbose=False)
            if out['status'] != 0:
                raise Exception('get_manifest_part_count failed, cmd status:'
                                + str(out['status']))
            output = out['output']
        else:
            with open(path) as m_file:
                output = m_file.read()
        xml = ElementTree.fromstring(output)
        return xml

    def get_manifest_part_count(self,
                                path,
                                machine=None,
                                local=False,
                                timeout=30):
        '''
        Attempt retrieve the part count value from a manifest file
        :param path: local or remote path to manifest file
        :param machine: the remote machine to read manifest file from
        :param local: boolean to determine if file is local
        :param timeout: timeout in second for reading in file
        :returns int count
        '''
        manifest_xml = self.get_manifest_obj(path=path,
                                             machine=machine,
                                             local=local,
                                             timeout=timeout)
        image = manifest_xml.find('image')
        parts = image.find('parts')
        part_count = parts.get('count')
        self.log.debug('get_manifest_part_count:' + str(path) +
                   ', count:' + str(part_count))
        return int(part_count)

    def get_manifest_image_name(self,
                                path,
                                machine=None,
                                local=False,
                                timeout=30):
        '''
        Attempts to read the image name from a manifest file
        :param path: local or remote path to manifest file
        :param machine: the remote machine to read manifest file from
        :param local: boolean to determine if file is local
        :param timeout: timeout in second for reading in file
        :returns string image name
        '''
        manifest_xml = self.get_manifest_obj(path=path,
                                             machine=machine,
                                             local=local,
                                             timeout=timeout)
        image = manifest_xml.find('image')
        name_elem = image.find('name')
        return name_elem.text

    def euca2ools_bundle_image(self,
                     path,
                     machine=None,
                     machine_credpath=None,
                     prefix=None,
                     kernel=None,
                     ramdisk=None,
                     access_key=None,
                     secret_key=None,
                     account_id=None,
                     ec2cert=None,
                     bootstrap_url=None,
                     block_device_mapping=None,
                     destination='/disk1/storage',
                     arch='x86_64',
                     debug=False,
                     user_context=None,
                     interbundle_timeout=120,
                     time_per_gig=None):
        '''
        Bundle an image on a 'machine'.
        where credpath to creds on machine
        '''
        self.status_log('Starting euca2ools_bundle_image at path:"{0}"'.format(path))
        time_per_gig = time_per_gig or self.time_per_gig
        machine = machine or self.worker_machine
        image_size = machine.get_file_size(path)/self.gig or 1
        timeout = time_per_gig * image_size
        cbargs = [timeout, interbundle_timeout, time.time(), 0, True]
        user_context = user_context or self.user_context
        if destination is None:
            destination = machine.sys('pwd')[0]
        freesize = machine.get_available(str(destination), (self.gig/self.kb))
        if (freesize < image_size):
            raise Exception("Not enough free space at:" + str(destination))
        ec2cert = ec2cert or self.ec2_cert_path
        bootstrap_url = bootstrap_url or self.bootstrap_url
        if not bootstrap_url and user_context:
            bootstrap_url = user_context.bootstrap_url
        access_key = access_key or self.access_key
        if not access_key and user_context:
            access_key = user_context.access_key
        secret_key = secret_key or self.secret_key
        if not secret_key and user_context:
            secret_key = user_context.secret_key
        account_id = account_id or self.account_id
        if not account_id and user_context:
            account_id = user_context.account_id

        #build our tools bundle-image command...
        cmdargs = ""
        if prefix:
            cmdargs = cmdargs + " --prefix " +str(prefix)
        if kernel:
            cmdargs = cmdargs + " --kernel "  +str(kernel)
        if ramdisk:
            cmdargs = cmdargs + " --ramdisk " +str(ramdisk)
        if block_device_mapping:
            cmdargs = cmdargs + " --block-device-mapping " + \
                      str(block_device_mapping)
        if destination:
            cmdargs = cmdargs + " --destination " + str(destination)
        if arch:
            cmdargs = cmdargs + " --arch " + str(arch)
        if account_id:
            cmdargs += " --user " + str(account_id)
        if bootstrap_url:
            cmdargs += " --bootstrap-url " + str(bootstrap_url)
        if ec2cert:
            cmdargs += " --ec2cert " + str(ec2cert)
        if debug:
            cmdargs = cmdargs + " --debug "

        cmdargs = cmdargs + " -i " + str(path)

        cmd = 'euca-bundle-image -I {0} -S {1} --user {2} {3}'\
            .format(access_key, secret_key, account_id, cmdargs)
        #execute the command
        out = machine.cmd(cmd, timeout=timeout, listformat=True,
                          cb = self._bundle_status_cb, cbargs=cbargs)
        if out['status'] != 0:
            raise Exception('bundle_image "' + str(path) +
                            '" failed. Errcode:' + str(out['status']))
        manifest = None
        for line in out['output']:
            line = str(line)
            if re.search("(Generating|Wrote) manifest",line):
                manifest = line.split()[2]
                break
        if manifest is None:
            raise Exception('Failed to find manifest from bundle_image:' +
                            str(path))
        self.log.debug('bundle_image:'+str(path)+'. manifest:'+str(manifest))
        return manifest

    def euca2ools_upload_bundle(self,
                      manifest,
                      access_key=None,
                      secret_key=None,
                      s3_url=None,
                      user_context=None,
                      machine=None,
                      bucketname=None,
                      debug=False,
                      interbundle_timeout=120,
                      timeout=0,
                      image_check_timeout=300,
                      uniquebucket=True,
                      retry=False):
        '''
        Bundle an image on a 'machine'.
        where credpath to creds on machine
        '''
        self.status_log('Starting euca2ools_upload_bundle for manifest:"{0}"'.format(manifest))
        machine = machine or self.worker_machine
        user_context = user_context or self.user_context
        s3_url = s3_url or self.s3_url
        if not s3_url and user_context:
            s3_url = user_context.s3_url
        access_key = access_key or self.access_key
        if not access_key and user_context:
            access_key = user_context.access_key
        secret_key = secret_key or self.secret_key
        if not secret_key and user_context:
            secret_key = user_context.secret_key
        if not access_key:
            raise ValueError('Could not determine access key for euca2ools_upload_bundle')
        if not secret_key:
            raise ValueError('Could not determine secret key for euca2ools_upload_bundle')
        cbargs = [timeout, interbundle_timeout, time.time(), 0, True]
        bname = ''
        cmdargs = ""
        manifest = str(manifest)
        upmanifest = None
        part_count = -1
        try:
            part_count = self.get_manifest_part_count(manifest, machine=machine)
        except:
            pass
        self.log.debug('Attempting to upload_bundle:' + str(manifest) +
                   ", bucketname:" + str(bucketname) + ", part_count:" +
                   str(part_count))
        if bucketname:
            bname = bucketname
            if uniquebucket:
                bname = self._get_unique_bucket_name(bname)
        else:
            #Use the image name found in the manifest as bucketname
            bname = self._generate_unique_bucket_name_from_manifest(
                manifest, unique=uniquebucket)
        self.log.debug('Using upload_bundle bucket name: '+str(bname))
        cmdargs = cmdargs + " -b " + str(bname)
        if s3_url:
            cmdargs += " --url " + str(s3_url)
        if retry:
            cmdargs += " --retry "
        if debug:
            cmdargs = cmdargs + " --debug "
        cmdargs = cmdargs + " -b " + str(bname) + " -m " +str(manifest)

        cmd = 'euca-upload-bundle -I ' + str(access_key) + ' -S ' + str(secret_key) + str(cmdargs)
        #execute upload-bundle command...
        out = machine.cmd(cmd, timeout=image_check_timeout, listformat=True,
                          cb=self._bundle_status_cb, cbargs=cbargs)
        if out['status'] != 0:
            raise RuntimeError('upload_bundle "' + str(manifest) +
                            '" failed. Errcode:' + str(out['status']))
        for line in out['output']:
            line = str(line)
            if re.search('Uploaded', line) and re.search('manifest', line):
                upmanifest = line.split().pop()
                break
        if upmanifest is None:
            raise RuntimeError('Failed to find upload manifest from '
                               'upload_bundle command')
        self.log.debug('upload_image:'+str(manifest)+'. manifest:'+str(upmanifest))
        return upmanifest

    def euca2ools_bundle_and_upload(self,
                                    file,
                                    arch,
                                    bucket,
                                    access_key=None,
                                    secret_key=None,
                                    s3_url=None,
                                    prefix=None,
                                    directory=None,
                                    kernel=None,
                                    ramdisk=None,
                                    block_device_map=[],
                                    product_codes=None,
                                    acl=None,
                                    location=None,
                                    ):
        raise NotImplemented('euca2ools_bundle_and_upload wrapper '
                             'not implemented yet')

    def euca2ools_register(self,
                           manifest,
                           name,
                           description=None,
                           access_key=None,
                           secret_key=None,
                           ec2_url=None,
                           arch=None,
                           kernel=None,
                           ramdisk=None,
                           root_device_name=None,
                           snapshot_id=None,
                           block_dev_map=[],
                           virtualization_type=None,
                           platform=None,
                           machine=None,
                           machine_credpath=None,
                           user_context=None):
        self.status_log('Starting euca2ools_register for manifest:"{0}", kernel:"{1}", ramdisk:"{2}"'
                    .format(manifest, kernel,ramdisk))
        user_context = user_context or self.user_context
        machine = machine or self.worker_machine
        access_key = access_key or self.access_key
        if not access_key and user_context:
            access_key = user_context.access_key
        secret_key = secret_key or self.secret_key
        if not secret_key and user_context:
            secret_key = user_context.secret_key
        ec2_url = ec2_url or self.ec2_url
        if not ec2_url and user_context:
            ec2_url = user_context.ec2_url
        if not access_key:
            self.log.warning('Could not determine access key for euca2ools_register')
        if not secret_key:
            self.log.warning('Could not determine secret key for euca2ools_register')
        if not manifest:
            self.log.warning('No manifest provided to register command! ')
        cmdargs = str(manifest) + " -n " + str(name)
        emi = None
        if description:
            cmdargs += ' -d ' + str(description)
        if arch:
            cmdargs += ' -a ' + str(arch)
        if kernel:
            cmdargs += ' --kernel ' + str(kernel)
        if ramdisk:
            cmdargs += ' --ramdisk ' + str(ramdisk)
        if root_device_name:
            cmdargs += ' --root-device-name ' + str(root_device_name)
        if snapshot_id:
            cmdargs += ' -s ' + str(snapshot_id)
        for dev in block_dev_map:
            cmdargs += ' -b ' + str(dev)
        if virtualization_type:
            cmdargs += ' --virtualization-type ' + str(virtualization_type)
        if platform:
            cmdargs += ' --platform ' + str(platform)
        if ec2_url:
            cmdargs += ' --url ' + str(ec2_url)


        cmd = 'euca-register {0} -I {1} -S {2}'.format(cmdargs, access_key, secret_key)
        out = machine.sys(cmd=cmd, code=0)
        for line in out:
            if re.search('IMAGE',line):
                emi = line.split().pop().strip()
        assert emi, 'Invalid emi value: "{0}"'.format(emi)
        return emi

    def euca2ools_download_bundle(self,
                                  bucket,
                                  manifest=None,
                                  prefix=None,
                                  directory=None,
                                  image_name=None,
                                  machine=None,
                                  access_key=None,
                                  secret_key=None,
                                  s3_url=None,
                                  ):
        machine = machine or self.worker_machine
        access_key = access_key or self.access_key
        secret_key = secret_key or self.secret_key
        s3_url = s3_url or self.s3_url
        if not access_key:
            self.log.warning('Could not determine access key for euca2ools_download_bundle')
        if not secret_key:
            self.log.warning('Could not determine secret key for euca2ools_download_bundle')
        cmdargs = " -b " + str(bucket)
        if manifest:
            cmdargs += " -m " + str(manifest)
        if prefix:
            cmdargs += " -p " + str(prefix)
        if directory:
            cmdargs += " -d " + str(directory)
        if s3_url:
            cmdargs += " --url " + str(s3_url)


        cmd = ('euca-download-bundle -I ' + access_key + ' -S ' + secret_key + str(cmdargs))
        out = machine.sys(cmd=cmd, code=0)
        # Complete this wrapper...
        raise NotImplemented('euca2ools_download_bundle wrapper '
                             'not implemented yet')

    def euca2ools_download_and_unbundle(self,
                                        bucket,
                                        manifest=None,
                                        prefix=None,
                                        directory=None,
                                        maxbytes=None,
                                        access_key=None,
                                        secret_key=None,
                                        s3_url=None,
                                        ):
        raise NotImplemented('euca2ools_download_and_unbundle wrapper'
                             ' not implemented yet')


    def euca2ools_import_volume(self,
                                import_file,
                                bucket,
                                zone,
                                format,
                                size=None,
                                presigned_manifest_url=None,
                                prefix=None,
                                days=None,
                                no_upload=None,
                                description=None,
                                s3_url=None,
                                ec2_url=None,
                                security_token=None,
                                machine=None,
                                access_key=None,
                                secret_key=None,
                                user_context=None,
                                region_domain=None,
                                misc=None):
        '''
        Note: Under normal conditions this will create a volume that may
        not be available to the returned task object immediately. The volume
        id will be available later by using task.update() on the returned
        task object. For this reason the volume is not being added to
        the tester's resources list here.
        '''
        machine = machine or self.worker_machine
        access_key = access_key or self.access_key
        secret_key = secret_key or self.secret_key
        user = user_context
        if not user:
            try:
                user = self.create_user_context(access_key=access_key, secret_key=secret_key,
                                                region_domain=region_domain, s3_url=s3_url,
                                                ec2_url=ec2_url)
            except Exception as UE:
                self.log.warning('Could not create user context for euca2ools_import_volume, '
                                 'proceeding in case this is a "negative test". Err:"{0}"'
                                 .format(UE))
        if user:
            ec2_url = ec2_url or user.ec2_url
            s3_url = s3_url or user.s3_url
            access_key = access_key or user.access_key
            secret_key = secret_key or user.secret_key

        if not access_key:
            self.log.warning('Could not determine access key for euca2ools_import_volume')
        if not secret_key:
            self.log.warning('Could not determine secret key for euca2ools_import_volume')
        cmdargs = str(import_file) + " -b " + str(bucket) + \
                  " -z " + str(zone) + " -f " + str(format) + \
                  " --show-empty-fields "
        emi = None
        if description:
            cmdargs += ' -d ' + str(description)
        if size:
            cmdargs += ' -s ' + str(size)
        if presigned_manifest_url:
            cmdargs += ' --manifest-url ' + str(presigned_manifest_url)
        if prefix:
            cmdargs += " --prefix " + str(prefix)
        if days:
            cmdargs += " -x " + str(days)
        if no_upload:
            cmdargs += " --no-upload "
        if s3_url:
            cmdargs += " --s3-url " + str(s3_url)
        if ec2_url:
            cmdargs += " -U " + str(ec2_url)
        if access_key:
            cmdargs += " -w " + access_key
        if secret_key:
            cmdargs += " -o " + secret_key
        if security_token:
            cmdargs += " --security-token " + str(security_token)
        if misc:
            cmdargs += misc

        cmd = 'euca-upload-import-volume ' + str(cmdargs)
        out = machine.sys(cmd=cmd, code=0)
        for line in out:
            lre = re.search('import-vol-\w{8}', line)
            if lre:
                taskid = lre.group()
        self.log.info('Import taskid:' + str(taskid))
        #check on system using describe...
        task = self.tc.user.ec2.get_conversion_task(taskid=taskid)
        assert task, 'Task not found in describe conversion tasks. "{0}"'\
            .format(str(taskid))
        return task


    def euca2ools_import_instance(self,
                                import_file,
                                bucket,
                                zone,
                                format,
                                instance_type,
                                arch,
                                platform,
                                size=None,
                                keypair=None,
                                presigned_manifest_url=None,
                                prefix=None,
                                days=None,
                                no_upload=None,
                                description=None,
                                group=None,
                                s3_url=None,
                                ec2_url=None,
                                image_size=None,
                                user_data=None,
                                user_data_file=None,
                                private_addr=None,
                                shutdown_behavior=None,
                                access_key=None,
                                secret_key=None,
                                region_domain=None,
                                user_context=None,
                                security_token=None,
                                machine=None,
                                machine_credpath=None,
                                misc=None,
                                time_per_gig=90):
        machine = machine or self.worker_machine
        access_key = access_key or self.access_key
        secret_key = secret_key or self.secret_key
        user = user_context
        if not user:
            try:
                user = self.create_user_context(access_key=access_key, secret_key=secret_key,
                                                region_domain=region_domain, s3_url=s3_url,
                                                ec2_url=ec2_url)
            except Exception as UE:
                self.log.warning('Could not create user context for euca2ools_import_instance, '
                                 'proceeding in case this is a "negative test". Err:"{0}"'
                                 .format(UE))
        if user:
            ec2_url = ec2_url or user.ec2_url
            s3_url = s3_url or user.s3_url
            access_key = access_key or user.access_key
            secret_key = secret_key or user.secret_key
        if not access_key:
            self.log.warning('Could not determine access key for euca2ools_import_instance')
        if not secret_key:
            self.log.warning('Could not determine secret key for euca2ools_import_instance')
        try:
            file_size = machine.get_file_size(import_file)
            gb = file_size/self.gig or 1
            timeout = gb * time_per_gig
        except:
            timeout = 300
        cmdargs = str(import_file) + " -b " + str(bucket) + \
                  " -z " + str(zone) + " -f " + str(format) + \
                  " -t " + str(instance_type) + " -a " + str(arch) + \
                  " -p " + str(platform) + \
                  " --show-empty-fields "
        emi = None
        if description:
            cmdargs += ' -d ' + str(description)
        if group:
            cmdargs += ' -g ' + str(group)
        if keypair:
            cmdargs += ' --key ' + str(keypair)
        if size:
            cmdargs += ' -s ' + str(size)
        if presigned_manifest_url:
            cmdargs += ' --manifest-url ' + str(presigned_manifest_url)
        if prefix:
            cmdargs += " --prefix " + str(prefix)
        if days:
            cmdargs += " -x " + str(days)
        if no_upload:
            cmdargs += " --no-upload "
        if s3_url:
            cmdargs += " --s3-url " + str(s3_url)
        if ec2_url:
            cmdargs += " -U " + str(ec2_url)
        if image_size:
            cmdargs += " --image-size " + str(image_size)
        if user_data:
            cmdargs += ' --user-data "' + str(user_data) +'"'
        if user_data_file:
            cmdargs += " --user-data-file " + str(user_data_file)
        if private_addr:
            cmdargs += " --private-ip-address"
        if shutdown_behavior:
            cmdargs += " --instance-initiated-shutdown-behavior " + \
                        str(shutdown_behavior)
        if access_key:
            cmdargs += " -I " + str(access_key)
        if secret_key:
            cmdargs += " -S " + str(secret_key)
        if security_token:
            cmdargs += " --security-token " + str(security_token)
        if misc:
            cmdargs += misc

        cmd = 'euca-import-instance ' + str(cmdargs)

        out = machine.sys(cmd=cmd, timeout=timeout, code=0)
        taskid = None
        for line in out:
            lre = re.search('import-i-\w{8}', line)
            if lre:
                taskid = lre.group()
        if not taskid:
            raise RuntimeError('Could not find a task id in output from cmd:'
                               + str(cmd))
        self.log.debug('Import taskid:' + str(taskid))
        #check on system using describe...
        if user:
            task = user.ec2.get_conversion_task(taskid=taskid)
            assert task, 'Task not found in describe conversion tasks. "{0}"'\
                .format(str(taskid))
        return taskid

    def _parse_euca2ools_conversion_task_output(self, output):
        splitlist = []
        retlist = []
        for line in output:
            splitlist.extend(line.split('\t'))
        if len(splitlist) % 2:
            raise ValueError("Conversion task output:\n" + str(output) +
                             '\nParsed euca2ools conversion task output does '
                             'not have an even number of fields to parse')
        #Build out a dict based upon the key value-ish format of the output
        task = ConversionTask()
        for i in xrange(0, len(splitlist)):
            if i % 2:
                task.endElement(name=lastkey, value=splitlist[i])
            else:
                lastkey = str(splitlist[i]).lower()
                if lastkey == 'tasktype':
                    if task:
                        retlist.append(task)
                    task = ConversionTask
        return retlist

    def _generate_unique_bucket_name_from_manifest(self, manifest, unique=True):
        mlist = str(manifest.replace('.manifest.xml', '')).split('/')
        basename = mlist[len(mlist)-1].replace('_', '').replace('.', '')
        if self.user_context:
            basename = "{0}-{1}".format((self.user_context.account_name or
                                         self.user_context.account_id), basename)
        if unique:
            return self._get_unique_bucket_name(basename)
        return basename

    def _get_unique_bucket_name(self, basename, id='test', start=0, user=None):
        bx=start
        bname = basename
        user = user or self.user_context
        while user.s3.get_bucket_by_name(bname) is not None:
            bx += 1
            bname = basename+str(id)+str(bx)
        return bname


    def create_working_dir_on_worker_machine(self, path, overwrite=False):
        path = str(path)
        if self.worker_machine.is_file(path):
            if not overwrite:
                raise Exception('Dir found on:' +
                                str(self.worker_machine.hostname) +
                                ',"' + path + '".\n' +
                                'Either remove conflicting files, '
                                'use "filepath" option or "overwrite"')
        else:
            self.worker_machine.sys('mkdir -p ' + path, code=0)


    def _bundle_status_cb(self, buf, cmdtimeout, parttimeout, starttime,
                         lasttime, check_image_stage):
        ret = SshCbReturn(stop=False)
        #if the over timeout or the callback interval has expired,
        # then return stop=true
        #interval timeout should not be hit due to the setting of the
        # timer value, but check here anyways

        if (cmdtimeout != 0) and ( int(time.time()-starttime) > cmdtimeout):
            self.log.debug('bundle_status_cb command timed out after ' +
                       str(cmdtimeout)+' seconds')
            ret.statuscode=-100
            ret.stop = True
            return ret
        if not check_image_stage:
            ret.settimer = parttimeout
            if (parttimeout != 0 and lasttime != 0) and \
                    (int(time.time()-lasttime) > parttimeout):
                self.log.debug('bundle_status_cb inter-part time out after ' +
                           str(parttimeout) + ' seconds')
                ret.statuscode=-100
                ret.stop = True
                return ret

        if re.search('[P|p]art:',buf):
            sys.stdout.write("\r\x1b[K"+str(buf).strip())
            sys.stdout.flush()
            check_image_stage=False
        else:
            #Print command output and write to ssh.cmd['output'] buffer
            ret.buf = buf
            self.log.debug(str(buf))
        #Command is still going, reset timer thread to intervaltimeout,
        # provide arguments for  next time this is called from ssh cmd.
        ret.stop = False
        ret.nextargs =[cmdtimeout, parttimeout, starttime,
                       time.time(), check_image_stage]
        return ret

    def create_emi(self,
                   image_url,
                   machine=None,
                   access_key=None,
                   secret_key=None,
                   ec2_url=None,
                   s3_url=None,
                   bucketname=None,
                   machine_credpath=None,
                   debug=False,
                   prefix=None,
                   kernel=None,
                   ramdisk=None,
                   architecture=None,
                   block_device_mapping=[],
                   destpath=None,
                   root_device_name=None,
                   description=None,
                   virtualization_type=None,
                   platform=None,
                   name=None,
                   interbundle_timeout=120,
                   upload_timeout=0,
                   uniquebucket=True,
                   wget_user=None,
                   wget_password=None,
                   wget_retryconn=True,
                   filepath=None,
                   bundle_manifest=None,
                   uploaded_manifest=None,
                   time_per_gig=300,
                   tagname=None,
                   overwrite=False,
                   user_context=None,
                   region_domain=None,
                   ):
        start = time.time()
        user = user_context
        if not user:
            try:
                user = self.create_user_context(access_key=access_key, secret_key=secret_key,
                                                region_domain=region_domain, s3_url=s3_url,
                                                ec2_url=ec2_url)
            except Exception as UE:
                self.log.error('Could not create user context for create_emi. Err:"{0}"'
                               .format(UE))
                raise UE
        if user:
            ec2_url = ec2_url or user.ec2_url
            s3_url = s3_url or user.s3_url
            access_key = access_key or user.access_key
            secret_key = secret_key or user.secret_key
        filesize = None
        destpath = destpath or self.destpath
        destpath = str(destpath)
        self.log.debug('create_emi_from_url:' + str(image_url) + ", starting...")

        if not destpath.endswith('/'):
            destpath += '/'
        if image_url:
            filename = str(image_url).split('/')[-1]
            destpath = destpath + str(filename.replace('.','_'))
            if filepath is None and bundle_manifest is None and uploaded_manifest is None:
                filepath = destpath + "/" + str(filename)
                self.create_working_dir_on_worker_machine(path=destpath,
                                                          overwrite=overwrite)

                self.log.debug('Downloading image to ' + str(machine) + ':' +
                               str(filepath) + ', image_url:' + str(image_url))
                filesize = self.wget_image(image_url,
                                           destpath=destpath,
                                           machine=machine,
                                           user=wget_user,
                                           password=wget_password,
                                           retryconn=wget_retryconn,
                                           time_per_gig=time_per_gig)

        self.status_log('create_emi_from_url: Image downloaded to machine, '
                    'now bundling image...')
        if bundle_manifest is None and uploaded_manifest is None:
            bundle_manifest = self.euca2ools_bundle_image(
                filepath,
                machine=machine,
                machine_credpath=machine_credpath,
                access_key=access_key,
                secret_key=secret_key,
                prefix=prefix,
                kernel=kernel,
                ramdisk=ramdisk,
                block_device_mapping=block_device_mapping,
                destination=destpath,
                debug=debug,
                interbundle_timeout=interbundle_timeout,
                time_per_gig=time_per_gig)

        self.status_log('create_emi_from_url: Image bundled, now uploading...')
        if uploaded_manifest is None:
            uploaded_manifest = self.euca2ools_upload_bundle(
                bundle_manifest,
                machine=machine,
                access_key=access_key,
                secret_key=secret_key,
                bucketname=bucketname,
                s3_url=s3_url,
                debug=debug,
                interbundle_timeout=interbundle_timeout,
                timeout=upload_timeout,
                uniquebucket=uniquebucket)

        self.status_log('create_emi_from_url: Now registering...')
        if name is None:
            name = uploaded_manifest.split('/').pop().rstrip('manifest.xml')
            name = "".join(re.findall(r"\w", name))
            name += '-' + str(int(time.time()))
        try:
            if user:
                user.ec2.get_emi(name='name')
        except:
            name += 'X'
        emi = self.euca2ools_register(
            manifest=uploaded_manifest,
            name=name,
            description=description,
            access_key=access_key,
            secret_key=secret_key,
            ec2_url=ec2_url,
            arch=architecture,
            kernel=kernel,
            ramdisk=ramdisk,
            root_device_name=root_device_name,
            block_dev_map=block_device_mapping,
            virtualization_type=virtualization_type,
            platform=platform)
        self.log.debug('euca2ools_register returned:"' + str(emi) +
                       '", now verify it exists on the cloud...')

        #Verify emi exists on the system, and convert to boto obj...
        emi = user.ec2.get_emi(emi)

        #Add tags that might have test use meaning...
        try:
            if filesize is not None:
                emi.add_tag('size', value= str(filesize))
            if image_url:
                emi.add_tag('source', value=(str(image_url)))
            emi.add_tag(tagname or 'eutester-created')
        except Exception, te:
            self.log.debug('Could not add tags to image:' + str(emi.id) +
                       ", err:" + str(te))
        elapsed= int(time.time()-start)
        self.status_log('create_emi_from_url: Done, image registered as:' +
                    str(emi.id) + ", after " + str(elapsed) + " seconds")
        return emi




from nephoria.testcontroller import TestController
from cloud_utils.log_utils import get_traceback, eulogger, red
from prettytable import PrettyTable
import time
from optparse import OptionParser
import resource
import signal
import sys





import __builtin__
openfiles = set()
oldfile = __builtin__.file

def printOpenFiles():
    print red("\n\n### %d OPEN FILES: [%s]\n\n" % (len(openfiles), ", ".join(f.x for f in openfiles)))

class newfile(oldfile):
    def __init__(self, *args):
        self.x = args[0]
        print red("### OPENING %s ###" % str(self.x))
        oldfile.__init__(self, *args)
        openfiles.add(self)
        printOpenFiles()

    def close(self):
        print red("### CLOSING %s ###" % str(self.x))
        oldfile.close(self)
        openfiles.remove(self)
        printOpenFiles()

oldopen = __builtin__.open
def newopen(*args):
    return newfile(*args)
__builtin__.file = newfile
__builtin__.open = newopen




class InstanceBatchTest():
    def __init__(self):
        self.name = 'InstanceBatchTest'
        parser = OptionParser('InstanceBatchTest')

        parser.add_option("-c", "--clc-ip", dest="clc", default=None,
                          help="CLC IP")
        parser.add_option("-p", "--password", dest="password", default='foobar',
                          help="clc ssh password")
        parser.add_option("-l", "--log-level", dest="log_level", default='DEBUG',
                          help="LOGLEVEL")
        parser.add_option("--instance-timeout", dest="instance_timeout", type='int', default=1200,
                          help="Seconds used as timeout in run-image/instance request")
        parser.add_option('--emi', type=str, default=None,
                                 help='Image id used to run VMs')
        parser.add_option('--keypair', type=str, default=None,
                                 help='EC2 Keypair name to use for VM connections, '
                                      'default:"InstanceBatchTestKey_<timestamp>"')
        parser.add_option('--zone', type=str, default=None,
                                 help='Name of availability zone to run VMs in')
        parser.add_option('--vmtype', type=str, default='t1.micro',
                                 help='Instance Vmtype to use')
        parser.add_option('--results-file', type=str, default=None,
                                 help='File to save results to')
        parser.add_option('--vm-count', type=int, default=10,
                                 help='Number of VMs to run per request')
        parser.add_option('--vm-max', type=int, default=500,
                                 help='Max or total number of VMs to run in this test')
        parser.add_option('--no-clean', default=False, action='store_true',
                          help="Do not terminate VMs during test")
        parser.add_option('--user', type=str, default='admin',
                                 help='Cloud username, default: "admin"')
        parser.add_option('--account', type=str, default='nephotest',
                                 help='Cloud account name, default:"nephotest"')


        self.args, pos = parser.parse_args()
        if not self.args.clc:
            raise ValueError('CLC must be provided. See --clc argument')
        self.log = eulogger.Eulogger('InstanceBatchTest', stdout_level=self.args.log_level)
        self.tc = TestController(self.args.clc,
                                 password=self.args.password,
                                 clouduser_account=self.args.account,
                                 clouduser_name=self.args.user,
                                 log_level=self.args.log_level)
        self.emi = self.tc.user.ec2.get_emi(emi=self.args.emi)
        self.vmtype = self.args.vmtype
        self.keyname = self.args.keypair or "InstanceBatchTestKey_{0}".format(int(time.time()))
        self.key = self.tc.user.ec2.get_keypair(key_name=self.keyname)
        self.group = self.tc.user.ec2.add_group('InstanceBatchTestGroup')
        self.tc.user.ec2.authorize_group(self.group, port=22, protocol='tcp')
        self.tc.user.ec2.authorize_group(self.group,  protocol='icmp', port=-1)
        self.tc.user.ec2.show_security_group(self.group)
        if not self.key:
            raise RuntimeError('Was not able to find or create key:"{0}". '
                               'If the key exists, it may not be in the local dir?')
        self.results = {}
        self.pt = PrettyTable(['RUN', 'START', 'TOTAL', 'NEW', 'ELAPSED'])
        self.last_kill_sig = 0
        self.kill = False


    def add_result(self, start_date, run_number, total, added, elapsed):
        self.results[run_number] = {'start_date': start_date,
                                    'total': total,
                                    'added': added,
                                    'elapsed': elapsed}
        self.pt.add_row([run_number, start_date, total, added, elapsed])
        self.log.info('')

    def run_batch_loop(self):
        error = ""
        error_count = 0
        self.log.info('Test start. Terminating all instances for user:{0}/{1}'
                      .format(self.tc.user.account_name, self.tc.user.user_name))
        if not self.args.no_clean:
            self.tc.user.ec2.connection.terminate_instances()
            #Monitor to terminated state
            self.tc.user.ec2.terminate_instances()

        existing_instances = self.tc.admin.ec2.get_instances(state='running')
        start_count = len(existing_instances)
        elapsed = 0
        run_number = 0

        added = 0
        ins_ids = []
        start_date = time.strftime("%Y-%m-%d %H:%M")
        self.add_result(start_date=start_date, run_number=run_number, total=start_count,
                        added=added, elapsed=elapsed)
        while (len(ins_ids) < self.args.vm_max) and not self.kill:
            try:
                printOpenFiles()
                run_number += 1
                start_date = time.strftime("%Y-%m-%d %H:%M")
                start_time = time.time()
                ins = self.tc.user.ec2.run_image(image=self.emi, keypair=self.key,
                                                 min=self.args.vm_count, max=self.args.vm_count,
                                                 zone=self.args.zone, vmtype=self.vmtype,
                                                 group=self.group,
                                                 timeout=self.args.instance_timeout,
                                                 )
                elapsed = time.time() - start_time
                added = len(ins)
                total = len(self.tc.admin.ec2.get_instances(state='running'))
                for i in ins:
                    ins_ids.append(i.id)
                    try:
                        i.log.close()
                        if i.ssh:
                            i.ssh.connection.close()
                            i.ssh.close()
                    except Exception as IE:
                        printOpenFiles()
                        self.log.warning('Error closing instances fds:"{0}"'.format(IE))
                    i = None
                ins = None
                self.add_result(start_date=start_date, run_number=run_number, total=total,
                                added=added, elapsed=elapsed)
                self.log.info('\n\nDone with iteration:"{0}", ran: "{1}". Now added:{2}/{3}\n\n'
                              .format(run_number, added, len(ins_ids), self.args.vm_max))
                time.sleep(1)
            except Exception as E:
                error_count += 1
                error = "{0}\n{1}".format(get_traceback(), E)
                self.log.error(error)
                if error_count > 1:
                    raise
            self.show_results()
        self.log.info('Done with test, ran: {0} instances'.format(len(ins_ids)))
        return error

    def show_results(self):
        self.log.info('\n{0}\n'.format(self.pt))
        if self.args.results_file:
            with open(self.args.results_file, 'w') as resfile:
                resfile.write('\n{0}\n'.format(self.pt))
                resfile.flush()

    def clean_method(self):
        if not self.args.no_clean:
            self.log.info('Terminating all instances for user:{0}/{1}'
                          .format(self.tc.user.account_name, self.tc.user.user_name))
            self.tc.user.ec2.terminate_instances()

if __name__ == "__main__":

    errors =[]
    test = InstanceBatchTest()
    try:
        # In the case we want to keep each instance connection open?...
        resource.setrlimit(resource.RLIMIT_NOFILE, (10 * test.args.vm_max ,-1))
    except Exception as RE:
        test.log.warning(red('Unable to set resource limit to:"{0}", err:"{1}"'
                             .format(10 * test.args.vm_max, RE)))

    def signal_handler(signal, frame):
        kill_it = False
        now = time.time()
        if now - test.last_kill_sig < 2:
            kill_it = True
        test.last_kill_sig = now
        sys.stderr.write(red('\n\nReceived SIGINT, dumping results. (Press ctrl+c twice quickly to end test now)\n\n'))
        sys.stderr.flush()
        test.show_results()
        if kill_it:
            sys.stderr.write(red('\n\nReceived SIGINT twice within 2 seconds killing test...!\n\n'))
            sys.stderr.flush()
            sys.exit(0)
        signal.signal(signal.SIGINT, signal_handler)

    for meth in [test.run_batch_loop, test.clean_method]:
        try:
           ret = meth()
           if ret:
               test.log.error(ret)
        except Exception as E:
            test.tc.log.error('{0}\nError in test:"{1}"'.format(get_traceback(), E))
            errors.append(E)
    test.show_results()
    printOpenFiles()
    exit(len(errors) and 1)

